/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_GL_X3DNODE_H
#define X3DTK_GL_X3DNODE_H

#include "X3DAbstractNode.h"
#include "X3D_X3DNode.h"
#include "GL_SceneGraphTypes.h"

namespace X3DTK {

/// namespace including the GL scene graph as well as its related classes.

namespace GL {

/*! \brief Base class declared in <b><X3DTK/GL/scenegraph.h></b> providing the GL implementation
 *  of X3D::X3DNode.
 *
 *  This is the root node of the GL derivation tree. 
 *
 *  \ingroup GLSceneGraph
 */
 
class X3DNode : public X3DAbstractNode
{
public:
  /// Constructor.
  X3DNode();
  /// Virtual destructor.
  virtual ~X3DNode();
  
  /// Sets the x3dReference.
  void setX3DReference(X3D::SFNode N);
  /// Returns the X3DNode equivalent.
  inline X3D::SFNode getX3DReference() const {return x3dReference;};
  
  /// Updates the attributes of the node.
  virtual void update();
  
protected:
  /// Copy constructor.
  X3DNode(const X3DNode &N);
  
  /// Pointer to the X3DNode equivalent.
  X3D::SFNode x3dReference;
};

}
}

#endif
