/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_GRAPHTRAVERSAL_H
#define X3DTK_GRAPHTRAVERSAL_H

#include "StateVariables.h"
#include "X3DTypes.h"

#include <list>
#include <typeinfo>
#include <iostream>

namespace X3DTK {

class Walker;
class Visitor; 
class X3DComponentWalker;
class X3DComponentVisitor;

/*! \brief Class declared in <b><X3DTK/kernel.h></b> defining the concept of tree traversal
 *  algorithm.
 
 *  This is also a StateVariables factory.
 *
 *  \ingroup processor
 */
 
class GraphTraversal
{
public:
  /// Constructor.
  GraphTraversal();
  /// Destructor.  
  virtual ~GraphTraversal();
  
  /// Sets a component walker. 
  void setComponentWalker(X3DComponentWalker *component);
  /// Sets a component visitor. 
  void setComponentVisitor(X3DComponentVisitor *component);
  /// Deletes the components when the object is deleted.
  void setAutoDeleteComponents(bool value);
  /// Traverses the scene.
  void traverse(SFAbstractNode N) const;
  
  /// Joins the content of D1 and D2.
  friend GraphTraversal *joinGraphTraversals(GraphTraversal *A1, GraphTraversal *A2);
  /// Merges the content of A1 and A2 and deletes A1 and A2.  
  friend GraphTraversal *mergeGraphTraversals(GraphTraversal *A1, GraphTraversal *A2);
  
  /// Gets the instance of StateVariables.
  template <class S>
  static S *getInstanceOf();
  
  /// Removes the instance of StateVariables.  
  template <class S>
  static void removeInstanceOf();
    
protected:
  /// Visitor.
  Visitor *visitor;     
  /// Walker.
  Walker *walker;

private:
  static std::list<StateVariables *> _stateVariablesList;  
};

}

#include "GraphTraversal.inl"

#endif
