/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_IMAGE_H
#define X3DTK_IMAGE_H

#include "X3DImageType.h"

namespace X3DTK {

/*! \brief Class declared in <b><X3DTK/kernel.h></b> providing an abstract factory 
 *  for the different image formats.
 *
 *  This class is defined to be user friendly for loading texture images in OpenGL.
 *
 *  \ingroup image
 */
 
class Image
{
public:
  /// Constructor.
  Image();
  /// Constructor with parameter.  
  Image(const char *fileName);
  /// Destructor.
  ~Image();
  
  /// Loads the file.
  bool load(const char *fileName);  
  /// Resizes the memory array to width*height.
  void resize(unsigned short width, unsigned short height);
  /// Resizes the memory array to a power of two and fills textureTransform, the 
  /// transformation to apply in OpenGL to draw the good texture.
  void resizeGL(float (& textureTransform)[16]);
  
  /// Gets the width.
  inline unsigned short getWidth() const {return (_imageType != 0 ? _imageType->getWidth() : 0);};
  /// Gets the height.
  inline unsigned short getHeight() const {return (_imageType != 0 ? _imageType->getHeight() : 0);};
  /// Gets the GL type of pixels.
  inline int getGLType() const {return (_imageType != 0 ? _imageType->getGLType() : 0);};
  /// Gets the data array.
  inline unsigned char* getData() const {return _imageType->getData();};
  
private:
  X3DImageType *_imageType;
};

}

#endif
