/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_CONNECTEDCOMCOMPUTER_H
#define X3DTK_MESH_CONNECTEDCOMCOMPUTER_H

#include "MESH_ConnectedComComputerStateVariables.h"
#include "X3DMultiPassProcessor.h"
#include "MESH_SceneGraphTypes.h"
#include "MESH_DFSGraphTraversal.h"
#include "MESH_ConnectedComComputerCoreVisitor.h"
#include "MESH_ConnectedComComputerMeshCoreVisitor.h"
#include "MESH_Mesh.h"
#include "MESH_MeshData.h"

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/facesdrawer.h></b> providing a processor that 
 *  draws the faces of the MESH scene graph.
 *
 *  \ingroup MESH
 */

template<class MData, class VData, class EData, class FData, bool readOnly>
class ConnectedComComputer : public X3DMultiPassProcessor
{
public:
  /// Constructor.
  ConnectedComComputer();
  /// Destructor.
  virtual ~ConnectedComComputer();
  
  /// Draws the faces.
  void compute(X3DNode *N);
  /// Sets the selected nodes to draw. If empty, then all the graph is drawn.
  void setSelectedNodes(const MFNode &selectedNodes);
  
protected:
  ConnectedComComputerStateVariables<MData, VData, EData, FData, readOnly> *stateVariables;
};

/// Default ConnectedComComputer.
typedef ConnectedComComputer<MeshData, VertexData, EdgeData, FaceData, true> DefConnectedComComputer;

}
}

#include "MESH_ConnectedComComputer.inl"

#endif

