namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
ConnectedComComputer<MData, VData, EData, FData, readOnly>::ConnectedComComputer()
: X3DMultiPassProcessor()
{
  stateVariables = GraphTraversal::getInstanceOf<ConnectedComComputerStateVariables<MData, VData, EData, FData, readOnly> >();

  graphTraversalMap["Vertex"] = new DFSGraphTraversal();
  graphTraversalMap["Vertex"]->setComponentVisitor(new ConnectedComComputerCoreVisitor<MData, VData, EData, FData, readOnly>());
  
  graphTraversalMap["Mesh"] = new DFSGraphTraversal();
  graphTraversalMap["Mesh"]->setComponentVisitor(new ConnectedComComputerMeshCoreVisitor<MData, VData, EData, FData, readOnly>());
}

template<class MData, class VData, class EData, class FData, bool readOnly>
ConnectedComComputer<MData, VData, EData, FData, readOnly>::~ConnectedComComputer()
{
  GraphTraversal::removeInstanceOf<ConnectedComComputerStateVariables<MData, VData, EData, FData, readOnly> >();

  delete graphTraversalMap["Vertex"];
  delete graphTraversalMap["Mesh"];
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void ConnectedComComputer<MData, VData, EData, FData, readOnly>::compute(X3DNode *N)
{
  stateVariables->init();
  graphTraversalMap["Vertex"]->traverse(N);
  stateVariables->finish(); 
/*  std::cout << "vs ok" << std::endl;
  graphTraversalMap["Mesh"]->traverse(N);
  stateVariables->finish(); 
  std::cout << "mesh ok" << std::endl;  */
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void ConnectedComComputer<MData, VData, EData, FData, readOnly>::setSelectedNodes(const MFNode &selectedNodes)
{
  stateVariables->setSelectedNodes(selectedNodes);
}

}
}  
