namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
ConnectedComComputerCoreVisitor<MData, VData, EData, FData, readOnly>::ConnectedComComputerCoreVisitor()
{
  // Enter and leave functions.
  defineEnterFunction(&ConnectedComComputerCoreVisitor::enterVertex);
  defineWalkOnFunction(&ConnectedComComputerCoreVisitor::walkOnShape);
  
  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<ConnectedComComputerStateVariables<MData, VData, EData, FData, readOnly> >();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
bool ConnectedComComputerCoreVisitor<MData, VData, EData, FData, readOnly>::walkOnShape(Shape *, SFAbstractNode child) const
{
  MFNode selectedNodes = stateVariables->getSelectedNodes();
  return (selectedNodes.empty() || (std::find(selectedNodes.begin(), selectedNodes.end(), child) != selectedNodes.end()));
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void ConnectedComComputerCoreVisitor<MData, VData, EData, FData, readOnly>::enterVertex(Vertex<VData, EData, FData, readOnly> *V) const
{
  if (!stateVariables->getNode(V))
  {
    // computing the list of Mesh parents of V.
    MFAbstractNode parentList = V->getParentList();
    std::list<Mesh<MData, VData, EData, FData, readOnly> *> meshList;
    for (MFAbstractNode::const_iterator it = parentList.begin(); it != parentList.end(); ++it)
    {
      if (dynamic_cast<Mesh<MData, VData, EData, FData, readOnly> *>(*it) != 0)
        meshList.push_back(static_cast<Mesh<MData, VData, EData, FData, readOnly> *>(*it));
    }
    
    // defining structures.
    typename std::vector<SFFace<FData, VData, EData, readOnly> *> stack;
    unsigned int stackSize = 0;
    for (typename std::list<Mesh<MData, VData, EData, FData, readOnly> *>::const_iterator it = meshList.begin(); it != meshList.end(); ++it)
      stackSize += (*it)->getFaces().size();
    stack.reserve(stackSize);
    
    typename std::list<Mesh<MData, VData, EData, FData, readOnly> *>::const_iterator itMesh = meshList.begin();
    if (itMesh == meshList.end()) return;
    
    typename Mesh<MData, VData, EData, FData, readOnly>::MFFace::const_iterator itFace = (*itMesh)->getFaces().begin();
    
    while (itMesh != meshList.end())
    {
      std::list<Mesh<MData, VData, EData, FData, readOnly> *> cmeshList;
      std::map<typename Mesh<MData, VData, EData, FData, readOnly>::SFFace *, Mesh<MData, VData, EData, FData, readOnly> *> faceMap;
      stack.push_back(*itFace);
      typename std::vector<SFFace<FData, VData, EData, readOnly> *>::const_iterator f = stack.end();
      --f;
      faceMap[*f] = *itMesh;
      if (std::find(cmeshList.begin(), cmeshList.end(), *itMesh) == cmeshList.end())
        cmeshList.push_back(*itMesh);
        
      while (f != stack.end())
      {
        const typename SFFace<FData, VData, EData, readOnly>::MFEdge &me = (*f)->getEdges();
        for (typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator e = me.begin(); e != me.end(); ++e)
        { 
          const typename SFEdge<EData, FData, VData, readOnly>::MFFace &mlf = (*e)->getLeftFaces();
          const typename SFEdge<EData, FData, VData, readOnly>::MFFace &mrf = (*e)->getRightFaces();
        
          for (typename SFEdge<EData, FData, VData, readOnly>::MFFace::const_iterator nf = mlf.begin(); nf != mlf.end(); ++nf)
          {
            if (faceMap.find(*nf) == faceMap.end())
            {
              // finding the Mesh                            
              for (typename std::list<Mesh<MData, VData, EData, FData, readOnly> *>::const_iterator it = meshList.begin(); it != meshList.end(); ++it)
              {
                const typename Mesh<MData, VData, EData, FData, readOnly>::MFFace &faces = (*it)->getFaces();
//                typename Mesh<MData, VData, EData, FData, readOnly>::MFFace::iterator itFF = faces.find(*nf);
                if (faces.find(*nf) != faces.end())
                {
                  faceMap[*nf] = *it;
                    
                  if (std::find(cmeshList.begin(), cmeshList.end(), *it) == cmeshList.end())
                    cmeshList.push_back(*it);

                  break;    
                }
              }  
              
              stack.push_back(*nf);
            }
          }
        
          for (typename SFEdge<EData, FData, VData, readOnly>::MFFace::const_iterator nf = mrf.begin(); nf != mrf.end(); ++nf)
          {
            if (faceMap.find(*nf) == faceMap.end())
            {
              // finding the Mesh                            
              for (typename std::list<Mesh<MData, VData, EData, FData, readOnly> *>::const_iterator it = meshList.begin(); it != meshList.end(); ++it)
              {
                const typename Mesh<MData, VData, EData, FData, readOnly>::MFFace &faces = (*it)->getFaces();
                if (faces.find(*nf) != faces.end())
                {
                  faceMap[*nf] = *it;
                    
                  if (std::find(cmeshList.begin(), cmeshList.end(), *it) == cmeshList.end())
                    cmeshList.push_back(*it);

                  break;    
                }
              }  

              stack.push_back(*nf);
            }
          }
        }
        ++f;
      }
      
      // creating new Meshs and Vertex.
      std::map<Mesh<MData, VData, EData, FData, readOnly> *, Mesh<MData, VData, EData, FData, readOnly> *> meshMap;
      Vertex<VData, EData, FData, readOnly> *newVertexNode = new Vertex<VData, EData, FData, readOnly>();
      for (typename std::list<Mesh<MData, VData, EData, FData, readOnly> *>::const_iterator it = cmeshList.begin(); it != cmeshList.end(); ++it)
      {
        Mesh<MData, VData, EData, FData, readOnly> *newMesh = new Mesh<MData, VData, EData, FData, readOnly>();
        newMesh->setVertex(newVertexNode);
        
        // filling Mesh datas.
        if (MData::template find<MESH::MeshNormalData>())
        { 
          newMesh->template ogetData<MESH::MeshNormalData>().setNormal((*it)->template ogetData<MESH::MeshNormalData>().hasNormal());
          newMesh->template ogetData<MESH::MeshNormalData>().setNormalPerVertex((*it)->template ogetData<MESH::MeshNormalData>().getNormalPerVertex());
        }  

        if (MData::template find<MESH::MeshColorData>())
        {
          newMesh->template ogetData<MESH::MeshColorData>().setColor((*it)->template ogetData<MESH::MeshColorData>().hasColor());
          newMesh->template ogetData<MESH::MeshColorData>().setColorPerVertex((*it)->template ogetData<MESH::MeshColorData>().getColorPerVertex());
          newMesh->template ogetData<MESH::MeshColorData>().setRGBA((*it)->template ogetData<MESH::MeshColorData>().getRGBA());
        }  

        if (MData::template find<MESH::MeshTexCoordData>())
        {
          newMesh->template ogetData<MESH::MeshTexCoordData>().setTexCoord((*it)->template ogetData<MESH::MeshTexCoordData>().hasTexCoord());
        }  

        meshMap[*it] = newMesh;
      }
      
      // iterating the faces of the connected component.
      std::map<unsigned int, unsigned int> newIndex;
      for (typename std::map<typename Mesh<MData, VData, EData, FData, readOnly>::SFFace *, Mesh<MData, VData, EData, FData, readOnly> *>::const_iterator it = faceMap.begin(); it != faceMap.end(); ++it)
      {
        MFInt32 faceCoordIndex;
        SFFace<FData, VData, EData, readOnly> *face = (*it).first;
        const typename SFFace<FData, VData, EData, readOnly>::MFEdge &me = face->getEdges();
        for (typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator e = me.begin(); e != me.end(); ++e)
        {
          SFVertex<VData, EData, FData, readOnly> *v = (*e)->getFromVertex();
          if (newIndex.find(v->getIndex()) == newIndex.end())
          {
            SFVertex<VData, EData, FData, readOnly> *newVertex = newVertexNode->createVertex();
            
            if (VData::template find<MESH::VertexPointData>())
              newVertex->template ogetData<MESH::VertexPointData>().setPoint(v->template ogetData<MESH::VertexPointData>().getPoint());
            
            newIndex[v->getIndex()] = newVertex->getIndex();
          }
          faceCoordIndex.push_back(newIndex[v->getIndex()]);
        }
        
        // filling face data.
        SFFace<FData, VData, EData, readOnly> *newFace = meshMap[(*it).second]->createFace(faceCoordIndex);
        if (FData::template find<MESH::FaceNormalData>())
          newFace->template ogetData<MESH::FaceNormalData>().setNormal(face->template ogetData<MESH::FaceNormalData>().getNormal());
        if (FData::template find<MESH::FaceColorData>())
          newFace->template ogetData<MESH::FaceColorData>().setColor(face->template ogetData<MESH::FaceColorData>().getColor());

        // filling vertex data.
        const typename SFFace<FData, VData, EData, readOnly>::MFEdge &nme = newFace->getEdges();
        typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator e = me.begin();
        typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator ne = nme.begin();
        while (e != me.end())
        {
          SFVertex<VData, EData, FData, readOnly> *v = (*e)->getFromVertex();
          SFVertex<VData, EData, FData, readOnly> *newVertex = (*ne)->getFromVertex();
          
          if (VData::template find<MESH::VertexNormalData>())
            newVertex->template ogetData<MESH::VertexNormalData>().setNormalOfFace(v->template ogetData<MESH::VertexNormalData>().getNormalOfFace(face), newFace);
          if (VData::template find<MESH::VertexColorData>())
            newVertex->template ogetData<MESH::VertexColorData>().setColorOfFace(v->template ogetData<MESH::VertexColorData>().getColorOfFace(face), newFace); 
          if (VData::template find<MESH::VertexTexCoordData>())
            newVertex->template ogetData<MESH::VertexTexCoordData>().setTexCoordOfFace(v->template ogetData<MESH::VertexTexCoordData>().getTexCoordOfFace(face), newFace);  
          ++e;
          ++ne;  
        }
      }
      
      // adding new Meshes to the scene graph.
      for (typename std::map<Mesh<MData, VData, EData, FData, readOnly> *, Mesh<MData, VData, EData, FData, readOnly> *>::const_iterator it = meshMap.begin(); it != meshMap.end(); ++it)
      {
        Mesh<MData, VData, EData, FData, readOnly> *mesh = (*it).first;
        Mesh<MData, VData, EData, FData, readOnly> *newMesh = (*it).second;
        
        MFAbstractNode parentList = mesh->getParentList();
        for (MFAbstractNode::const_iterator itParent = parentList.begin(); itParent != parentList.end(); ++itParent)
        {
          SFAbstractNode cparent = (*itParent)->clone();
          cparent->setChild(newMesh);
          
          MFAbstractNode pparentList = (*itParent)->getParentList();
          for (MFAbstractNode::const_iterator itPParent = pparentList.begin(); itPParent != pparentList.end(); ++itPParent)
            (*itPParent)->addChild(cparent);
        }
      }

      // finding the next face.
      while ((itMesh != meshList.end()) && (std::find(stack.begin(), stack.end(), *itFace) != stack.end()))
      {
        ++itFace;
        if (itFace == (*itMesh)->getFaces().end())
        {
          ++itMesh;
          if (itMesh != meshList.end())
            itFace = (*itMesh)->getFaces().begin();
        }    
      }
    }

    // remove old meshes.
    for (typename std::list<Mesh<MData, VData, EData, FData, readOnly> *>::const_iterator it = meshList.begin(); it != meshList.end(); ++it)
    {
      MFAbstractNode parentList = (*it)->getParentList();
      for (MFAbstractNode::const_iterator itParent = parentList.begin(); itParent != parentList.end(); ++itParent)
      {
        MFAbstractNode pparentList = (*itParent)->getParentList();
        for (MFAbstractNode::const_iterator itPParent = pparentList.begin(); itPParent != pparentList.end(); ++itPParent)
        {
          (*itPParent)->removeChild(*itParent);
          X3DProcessor::getInstanceOf<MemReleaser>()->release(*itParent);
        }
      }
    }
    stateVariables->addNode(V);
  }
}

}
}  
