namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
ConnectedComComputerMeshCoreVisitor<MData, VData, EData, FData, readOnly>::ConnectedComComputerMeshCoreVisitor()
{
  // Enter and leave functions.
  defineEnterFunction(&ConnectedComComputerMeshCoreVisitor::enterMesh);
  
  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<ConnectedComComputerStateVariables<MData, VData, EData, FData, readOnly> >();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void ConnectedComComputerMeshCoreVisitor<MData, VData, EData, FData, readOnly>::enterMesh(Mesh<MData, VData, EData, FData, readOnly> *M) const
{
  std::cout << "entering " << M << std::endl;
  
  if (!stateVariables->getNode(M))
  {
    std::cout << "processing " << M << std::endl;
    
    // computing the list of Mesh brothers of M.
    MFAbstractNode parentList = M->getVertex()->getParentList();
    
    // defining structures.
    typename std::vector<SFFace<FData, VData, EData, readOnly> *> stack;
    const typename Mesh<MData, VData, EData, FData, readOnly>::MFFace &faces = M->getFaces();
    stack.reserve(faces.size());

    for (typename Mesh<MData, VData, EData, FData, readOnly>::MFFace::const_iterator it = faces.begin(); it != faces.end(); ++it)
    {
      std::cout << *it << " -> " << **it << std::endl;
    }  
    
    typename Mesh<MData, VData, EData, FData, readOnly>::MFFace::const_iterator itFace = M->getFaces().begin();
    
    while (itFace != faces.end())
    {
//      std::cout << "new connected component" << std::endl;
      
      std::set<SFFace<FData, VData, EData, readOnly> *> faceSet;
      stack.push_back(*itFace);
      typename std::vector<SFFace<FData, VData, EData, readOnly> *>::const_iterator f = stack.end();
      --f;
      faceSet.insert(*itFace);  
//      std::cout << "face = " << **f  << ", " << *itMesh << std::endl;
      
      std::cout << "new component" << std::endl;
      
      while (f != stack.end())
      {
        const typename SFFace<FData, VData, EData, readOnly>::MFEdge &me = (*f)->getEdges();
        std::cout << "&f = " << *f << std::endl;
        std::cout << "f = " << **f << std::endl;
        for (typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator e = me.begin(); e != me.end(); ++e)
        { 
          std::cout << "edge = " << **e << std::endl;
          
          const typename SFEdge<EData, FData, VData, readOnly>::MFFace &mlf = (*e)->getLeftFaces();
          const typename SFEdge<EData, FData, VData, readOnly>::MFFace &mrf = (*e)->getRightFaces();
        
          for (typename SFEdge<EData, FData, VData, readOnly>::MFFace::const_iterator nf = mlf.begin(); nf != mlf.end(); ++nf)
          {
            std::cout << "testing " << **nf << std::endl;
            if (faceSet.find(*nf) == faceSet.end() && (faces.find(*nf) != faces.end()))
            {
              std::cout << "inserting " << **nf << std::endl;
              faceSet.insert(*nf); 
              std::cout << "pushing " << *nf << std::endl;
              stack.push_back(*nf);
            }
          }
        
          for (typename SFEdge<EData, FData, VData, readOnly>::MFFace::const_iterator nf = mrf.begin(); nf != mrf.end(); ++nf)
          {
            std::cout << "testing " << **nf << std::endl;          
            if (faceSet.find(*nf) == faceSet.end() && (faces.find(*nf) != faces.end()))
            {
              std::cout << "inserting " << **nf << std::endl;            
              faceSet.insert(*nf); 
              std::cout << "pushing " << *nf << std::endl;                
              stack.push_back(*nf);
            }
          }
        }
        ++f;
      }

      // creating new Meshs and Vertex.
      Mesh<MData, VData, EData, FData, readOnly> *newMesh = new Mesh<MData, VData, EData, FData, readOnly>();
      newMesh->setChild(M->getVertex());

      // filling Mesh datas.
      if (MData::template find<MESH::MeshNormalData>())
      { 
        newMesh->template ogetData<MESH::MeshNormalData>().setNormal(M->template ogetData<MESH::MeshNormalData>().hasNormal());
        newMesh->template ogetData<MESH::MeshNormalData>().setNormalPerVertex(M->template ogetData<MESH::MeshNormalData>().getNormalPerVertex());
      }  

      if (MData::template find<MESH::MeshColorData>())
      {
        newMesh->template ogetData<MESH::MeshColorData>().setColor(M->template ogetData<MESH::MeshColorData>().hasColor());
        newMesh->template ogetData<MESH::MeshColorData>().setColorPerVertex(M->template ogetData<MESH::MeshColorData>().getColorPerVertex());
        newMesh->template ogetData<MESH::MeshColorData>().setRGBA(M->template ogetData<MESH::MeshColorData>().getRGBA());
      }  

      if (MData::template find<MESH::MeshTexCoordData>())
      {
        newMesh->template ogetData<MESH::MeshTexCoordData>().setTexCoord(M->template ogetData<MESH::MeshTexCoordData>().hasTexCoord());
      }

      // iterating the faces of the connected component.
      for (typename std::set<SFFace<FData, VData, EData, readOnly> *>::const_iterator it = faceSet.begin(); it != faceSet.end(); ++it)
      {
        MFInt32 faceCoordIndex;
        SFFace<FData, VData, EData, readOnly> *face = *it;
/*        std::cout << "&face = " << face << std::endl;  
        std::cout << "face = " << *face << std::endl;  */
        
        const typename SFFace<FData, VData, EData, readOnly>::MFEdge &me = face->getEdges();
//        std::cout << "3c" << std::endl;  
        for (typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator e = me.begin(); e != me.end(); ++e)
          faceCoordIndex.push_back((*e)->getFromVertex()->getIndex());
//      std::cout << "3d" << std::endl;  

        SFFace<FData, VData, EData, readOnly> *newFace = newMesh->createFace(faceCoordIndex);
        if (FData::template find<MESH::FaceNormalData>())
          newFace->template ogetData<MESH::FaceNormalData>().setNormal(face->template ogetData<MESH::FaceNormalData>().getNormal());
        if (FData::template find<MESH::FaceColorData>())
          newFace->template ogetData<MESH::FaceColorData>().setColor(face->template ogetData<MESH::FaceColorData>().getColor());
//        std::cout << "3c" << std::endl;  
        
        // filling vertex data.
        const typename SFFace<FData, VData, EData, readOnly>::MFEdge &nme = newFace->getEdges();
        typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator e = me.begin();
        typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator ne = nme.begin();
        while (e != me.end())
        {
          SFVertex<VData, EData, FData, readOnly> *v = (*e)->getFromVertex();
          SFVertex<VData, EData, FData, readOnly> *newVertex = (*ne)->getFromVertex();
          
          if (VData::template find<MESH::VertexNormalData>())
            newVertex->template ogetData<MESH::VertexNormalData>().setNormalOfFace(v->template ogetData<MESH::VertexNormalData>().getNormalOfFace(face), newFace);
          if (VData::template find<MESH::VertexColorData>())
            newVertex->template ogetData<MESH::VertexColorData>().setColorOfFace(v->template ogetData<MESH::VertexColorData>().getColorOfFace(face), newFace); 
          if (VData::template find<MESH::VertexTexCoordData>())
            newVertex->template ogetData<MESH::VertexTexCoordData>().setTexCoordOfFace(v->template ogetData<MESH::VertexTexCoordData>().getTexCoordOfFace(face), newFace);  
          ++e;
          ++ne;  
        }


      }

//      std::cout << "4" << std::endl;  

      // adding new Meshes to the scene graph.
      MFAbstractNode parentList = M->getParentList();
      for (MFAbstractNode::const_iterator itParent = parentList.begin(); itParent != parentList.end(); ++itParent)
      {
        SFAbstractNode cparent = (*itParent)->clone();
        cparent->setChild(newMesh);
          
        MFAbstractNode pparentList = (*itParent)->getParentList();
        for (MFAbstractNode::const_iterator itPParent = pparentList.begin(); itPParent != pparentList.end(); ++itPParent)
          (*itPParent)->addChild(cparent);
      }

//      std::cout << "5" << std::endl;  

      // finding the next face.
      while ((itFace != faces.end()) && (std::find(stack.begin(), stack.end(), *itFace) != stack.end()))
        ++itFace;
    }

//    std::cout << "6" << std::endl;  
    // remove old mesh.
    parentList = M->getParentList();
    for (MFAbstractNode::const_iterator itParent = parentList.begin(); itParent != parentList.end(); ++itParent)
    {
      MFAbstractNode pparentList = (*itParent)->getParentList();
      for (MFAbstractNode::const_iterator itPParent = pparentList.begin(); itPParent != pparentList.end(); ++itPParent)
      {
        (*itPParent)->removeChild(*itParent);
        std::cout << "trying to release " << (*itParent)->getTypeName() << " " << *itParent << std::endl;
//        X3DProcessor::getInstanceOf<MemReleaser>()->release(*itParent);
      }
    }
    
//    std::cout << "7" << std::endl;  
    
    stateVariables->addNode(M);
  }
}

}
}  
