namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
EdgesDrawer<MData, VData, EData, FData, readOnly>::EdgesDrawer() 
: X3DOnePassProcessor()
{
  stateVariables = GraphTraversal::getInstanceOf<EdgesDrawerStateVariables<MData, VData, EData, FData, readOnly> >();

  graphTraversal = new DFSGraphTraversal();
  graphTraversal->setComponentVisitor(new EdgesDrawerCoreVisitor<MData, VData, EData, FData, readOnly>());
}

template<class MData, class VData, class EData, class FData, bool readOnly> 
EdgesDrawer<MData, VData, EData, FData, readOnly>::~EdgesDrawer()
{
  GraphTraversal::removeInstanceOf<EdgesDrawerStateVariables<MData, VData, EData, FData, readOnly> >();
  delete graphTraversal;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void EdgesDrawer<MData, VData, EData, FData, readOnly>::draw(X3DNode *N, bool selection)
{
  if (VData::template find<MESH::VertexPointData>())
  {
    stateVariables->setSelectionMode(selection);
    stateVariables->init();
    graphTraversal->traverse(N);
    stateVariables->finish(); 
  }  
}

template<class MData, class VData, class EData, class FData, bool readOnly>
Mesh<MData, VData, EData, FData, readOnly> *EdgesDrawer<MData, VData, EData, FData, readOnly>::getMesh(int i) const 
{
  return stateVariables->getMesh(i);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const SFMatrix34f &EdgesDrawer<MData, VData, EData, FData, readOnly>::getMatrix(int i) const 
{
  return stateVariables->getMatrix(i);
}

}
}  
