namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
EdgesDrawerCoreVisitor<MData, VData, EData, FData, readOnly>::EdgesDrawerCoreVisitor()
{
  // Enter and leave functions.
  defineEnterFunction(&EdgesDrawerCoreVisitor::enterTransform);
  defineEnterFunction(&EdgesDrawerCoreVisitor::enterMesh);
  defineLeaveFunction(&EdgesDrawerCoreVisitor::leaveTransform);

  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<EdgesDrawerStateVariables<MData, VData, EData, FData, readOnly> >();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void EdgesDrawerCoreVisitor<MData, VData, EData, FData, readOnly>::enterMesh(Mesh<MData, VData, EData, FData, readOnly> *M) const
{
  stateVariables->storeMesh(M);
  int meshId = stateVariables->getCurrentMeshId();

  if (stateVariables->getSelectionMode()) 
    glPushName(meshId);

  glColor3f(1.0f, 0.9f, 0.8f);
  glLineWidth(1.0f);
  glDisable(GL_LIGHTING);  

  const typename Mesh<MData, VData, EData, FData, readOnly>::MFEdge &edges = M->getEdges();

  int index = 0;
  for (typename Mesh<MData, VData, EData, FData, readOnly>::MFEdge::const_iterator e = edges.begin(), end = edges.end() ; e != end; ++e)
  {
    if (stateVariables->getSelectionMode())
      glPushName(index++);

    if ((*e)->isBoundary())
      glColor3f(0.3f, 0.3f, 0.9f);
    else
    {
      if ((*e)->isNonManifold())
        glColor3f(0.3f, 0.9f, 0.3f);
      else
        glColor3f(0.7f, 0.7f, 0.7f);
    }  

    glBegin(GL_LINES);
    SFPoint3f point = (*e)->getFromVertex()->template ogetData<MESH::VertexPointData>().getPoint();
    glVertex3fv(point.f_data());
    point = (*e)->getToVertex()->template ogetData<MESH::VertexPointData>().getPoint();
    glVertex3fv(point.f_data());
    glEnd();
    
    if ((*e)->isNonManifold())
    {
      glEnable(GL_LIGHTING);      
      glColor3f(0.7f, 1.0f, 0.7f);
      const typename SFEdge<EData, FData, VData, readOnly>::MFFace &lfaces = (*e)->getLeftFaces();
      for (typename SFEdge<EData, FData, VData, readOnly>::MFFace::const_iterator f = lfaces.begin(), end = lfaces.end() ; f != end; ++f)
      {
        glBegin(GL_POLYGON);
        SFVec3f normal = (*f)->template ogetData<MESH::FaceNormalData>().getNormal();
        const typename SFFace<FData, VData, EData, readOnly>::MFEdge &edges = (*f)->getEdges();
        for (typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator ep = edges.begin(), end = edges.end() ; ep != end; ++ep)
        {
          SFPoint3f point = (*ep)->getFromVertex()->template ogetData<MESH::VertexPointData>().getPoint();
          glNormal3fv(normal.f_data());
          glVertex3fv(point.f_data());
        }
        glEnd();
      }
      glEnable(GL_LIGHTING);      
      glColor3f(0.2f, 0.6f, 0.2f);
      const typename SFEdge<EData, FData, VData, readOnly>::MFFace &rfaces = (*e)->getRightFaces();
      for (typename SFEdge<EData, FData, VData, readOnly>::MFFace::const_iterator f = rfaces.begin(), end = rfaces.end() ; f != end; ++f)
      {
        glBegin(GL_POLYGON);
        SFVec3f normal = (*f)->template ogetData<MESH::FaceNormalData>().getNormal();
        const typename SFFace<FData, VData, EData, readOnly>::MFEdge &edges = (*f)->getEdges();
        for (typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator ep = edges.begin(), end = edges.end() ; ep != end; ++ep)
        {
          SFPoint3f point = (*ep)->getFromVertex()->template ogetData<MESH::VertexPointData>().getPoint();
          glNormal3fv(normal.f_data());
          glVertex3fv(point.f_data());
        }
        glEnd();
      }
      
      glDisable(GL_LIGHTING);        
    }
          
    if (stateVariables->getSelectionMode())
      glPopName();
  }
  if (stateVariables->getSelectionMode())
    glPopName();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void EdgesDrawerCoreVisitor<MData, VData, EData, FData, readOnly>::enterTransform(Transform *T) const
{
  glMatrixMode(GL_MODELVIEW);
  stateVariables->pushMatrix(T->getTransform());
  glPushMatrix();
  glMultMatrixf(T->getTransform().toFloat16());
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void EdgesDrawerCoreVisitor<MData, VData, EData, FData, readOnly>::leaveTransform(Transform *) const
{
  glMatrixMode(GL_MODELVIEW);
  stateVariables->popMatrix();
  glPopMatrix();
}

}
}
