namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
EdgesDrawerStateVariables<MData, VData, EData, FData, readOnly>::EdgesDrawerStateVariables()
: StateVariables(), _selectionMode(false)
{
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void EdgesDrawerStateVariables<MData, VData, EData, FData, readOnly>::init()
{
  _matrixStack.push_front(SFMatrix34f::identity);
  _matrixVector.clear();
  _meshVector.clear();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void EdgesDrawerStateVariables<MData, VData, EData, FData, readOnly>::finish()
{
  _matrixStack.pop_front();
  if (_matrixStack.size() != 0)
  {
    cx3d << "EdgesDrawerStateVariables::finish: Non empty matrix stack at the end of EdgesDrawer traversal" << std::endl;
    _matrixStack.clear();
  }
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void EdgesDrawerStateVariables<MData, VData, EData, FData, readOnly>::pushMatrix(const SFMatrix34f &transformation)
{
  _matrixStack.push_front(_matrixStack.front()*transformation);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void EdgesDrawerStateVariables<MData, VData, EData, FData, readOnly>::popMatrix()
{
  _matrixStack.pop_front();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
int EdgesDrawerStateVariables<MData, VData, EData, FData, readOnly>::getCurrentMeshId()
{
  return _matrixVector.size() - 1;
}
  
template<class MData, class VData, class EData, class FData, bool readOnly>
void EdgesDrawerStateVariables<MData, VData, EData, FData, readOnly>::storeMesh(Mesh<MData, VData, EData, FData, readOnly> *mesh)
{
  _matrixVector.push_back(_matrixStack.front());
  _meshVector.push_back(mesh);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
Mesh<MData, VData, EData, FData, readOnly> *EdgesDrawerStateVariables<MData, VData, EData, FData, readOnly>::getMesh(int i) const 
{
  return _meshVector[i];
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const SFMatrix34f &EdgesDrawerStateVariables<MData, VData, EData, FData, readOnly>::getMatrix(int i) const 
{
  return _matrixVector[i];
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void EdgesDrawerStateVariables<MData, VData, EData, FData, readOnly>::setSelectionMode(bool selectionMode) 
{
  _selectionMode = selectionMode;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
bool EdgesDrawerStateVariables<MData, VData, EData, FData, readOnly>::getSelectionMode() const 
{
  return _selectionMode;
}

}
}
