namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
FacesDrawer<MData, VData, EData, FData, readOnly>::FacesDrawer()
: X3DOnePassProcessor()
{
  stateVariables = GraphTraversal::getInstanceOf<FacesDrawerStateVariables<MData, VData, EData, FData, readOnly> >();

  graphTraversal = new DFSGraphTraversal();
  graphTraversal->setComponentVisitor(new FacesDrawerCoreVisitor<MData, VData, EData, FData, readOnly>());
}

template<class MData, class VData, class EData, class FData, bool readOnly>
FacesDrawer<MData, VData, EData, FData, readOnly>::~FacesDrawer()
{
  GraphTraversal::removeInstanceOf<FacesDrawerStateVariables<MData, VData, EData, FData, readOnly> >();
  delete graphTraversal;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void FacesDrawer<MData, VData, EData, FData, readOnly>::changeOneColorPerMesh()
{
  stateVariables->changeOneColorPerMesh();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void FacesDrawer<MData, VData, EData, FData, readOnly>::draw(X3DNode *N, bool selection)
{
  if (VData::template find<MESH::VertexPointData>())
  {
    stateVariables->setSelectionMode(selection);
    stateVariables->init();
    graphTraversal->traverse(N);
    stateVariables->finish(); 
  }  
}

template<class MData, class VData, class EData, class FData, bool readOnly>
Mesh<MData, VData, EData, FData, readOnly> *FacesDrawer<MData, VData, EData, FData, readOnly>::getMesh(int i) const 
{
  return stateVariables->getMesh(i);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const SFMatrix34f &FacesDrawer<MData, VData, EData, FData, readOnly>::getMatrix(int i) const 
{
  return stateVariables->getMatrix(i);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
typename Mesh<MData, VData, EData, FData, readOnly>::SFFace *FacesDrawer<MData, VData, EData, FData, readOnly>::getFaceOfMesh(int i, int j) const 
{
  return (stateVariables->getMesh(j)->getFaces())[i];
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void FacesDrawer<MData, VData, EData, FData, readOnly>::setRenderingMode(FaceMode mode)
{
  stateVariables->setRenderingMode(mode);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void FacesDrawer<MData, VData, EData, FData, readOnly>::setSelectedNodes(const MFNode &selectedNodes)
{
  stateVariables->setSelectedNodes(selectedNodes);
}

}
}  
