namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
FacesDrawerStateVariables<MData, VData, EData, FData, readOnly>::FacesDrawerStateVariables()
: StateVariables(), _selectionMode(false), _mode(IFS)
{
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void FacesDrawerStateVariables<MData, VData, EData, FData, readOnly>::init()
{
  _matrixStack.push_front(SFMatrix34f::identity);
  _matrixVector.clear();
  _meshVector.clear();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void FacesDrawerStateVariables<MData, VData, EData, FData, readOnly>::finish()
{
  _matrixStack.pop_front();
  if (_matrixStack.size() != 0)
  {
    cx3d << "FacesDrawerStateVariables::finish: Non empty matrix stack at the end of FacesDrawer traversal" << std::endl;
    _matrixStack.clear();
  }
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void FacesDrawerStateVariables<MData, VData, EData, FData, readOnly>::pushMatrix(const SFMatrix34f &transformation)
{
  _matrixStack.push_front(_matrixStack.front()*transformation);
}  

template<class MData, class VData, class EData, class FData, bool readOnly>
void FacesDrawerStateVariables<MData, VData, EData, FData, readOnly>::popMatrix()
{
  _matrixStack.pop_front(); 
}

template<class MData, class VData, class EData, class FData, bool readOnly>
int FacesDrawerStateVariables<MData, VData, EData, FData, readOnly>::getCurrentMeshId()
{
  return _matrixVector.size() - 1;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void FacesDrawerStateVariables<MData, VData, EData, FData, readOnly>::storeMesh(Mesh<MData, VData, EData, FData, readOnly> *mesh)
{
  _matrixVector.push_back(_matrixStack.front());
  _meshVector.push_back(mesh);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
Mesh<MData, VData, EData, FData, readOnly> *FacesDrawerStateVariables<MData, VData, EData, FData, readOnly>::getMesh(int i) const 
{
  return _meshVector[i];
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const SFMatrix34f &FacesDrawerStateVariables<MData, VData, EData, FData, readOnly>::getMatrix(int i) const 
{
  return _matrixVector[i];
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void FacesDrawerStateVariables<MData, VData, EData, FData, readOnly>::setSelectionMode(bool selectionMode) 
{
  _selectionMode = selectionMode;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
bool FacesDrawerStateVariables<MData, VData, EData, FData, readOnly>::getSelectionMode() const 
{
  return _selectionMode;
}
  
template<class MData, class VData, class EData, class FData, bool readOnly>
void FacesDrawerStateVariables<MData, VData, EData, FData, readOnly>::setRenderingMode(FaceMode mode)
{
  _mode =  mode;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
FaceMode FacesDrawerStateVariables<MData, VData, EData, FData, readOnly>::getRenderingMode() const 
{
  return _mode;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void FacesDrawerStateVariables<MData, VData, EData, FData, readOnly>::setSelectedNodes(const MFNode &selectedNodes)
{
  _selectedNodes = selectedNodes;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
MFNode FacesDrawerStateVariables<MData, VData, EData, FData, readOnly>::getSelectedNodes() const
{
  return _selectedNodes;
}

}
}
