////////////////////////////////////////////////////////////////////////////////
//                            MESH_Mesh.inl                                   //
////////////////////////////////////////////////////////////////////////////////

namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
Mesh<MData, VData, EData, FData, readOnly>::Mesh()
: X3DMeshNode()
{
  defineTypeName(typeid(*this).name());
}

template<class MData, class VData, class EData, class FData, bool readOnly>
SFNode Mesh<MData, VData, EData, FData, readOnly>::clone() const
{
  return new Mesh(*this);  
}

template<class MData, class VData, class EData, class FData, bool readOnly>
Mesh<MData, VData, EData, FData, readOnly>::~Mesh() 
{
  removeParentFromChild(this, vertex);   

  // delete edges
  for (typename MFEdge::iterator it = _edges.begin(); it != _edges.end(); ++it)
  {
    delete (*it)->_symetric;
    delete (*it)->_edgeContent;
    delete (*it);    
  } 
   
  // delete faces
  for (typename MFFace::iterator it = _faces.begin(); it != _faces.end(); ++it)
    delete (*it); 
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void Mesh<MData, VData, EData, FData, readOnly>::setVertex(Vertex<VData, EData, FData, readOnly> *vertex)
{
  removeParentFromChild(this, this->vertex);  
  this->vertex = vertex;
  addParentToChild(this, this->vertex);  
}

template<class MData, class VData, class EData, class FData, bool readOnly>
MFAbstractNode Mesh<MData, VData, EData, FData, readOnly>::getChildrenList() const
{
  MFAbstractNode childrenList;
  if (vertex != 0)
    childrenList.push_back(vertex);
  
  return childrenList;   
}

template<class MData, class VData, class EData, class FData, bool readOnly>
bool Mesh<MData, VData, EData, FData, readOnly>::addChild(const SFAbstractNode &N)
{
  if (dynamic_cast<Vertex<VData, EData, FData, readOnly> *>(N) != 0)
  {
    if (vertex != 0)
      return false;
    
    setVertex(static_cast<Vertex<VData, EData, FData, readOnly> *>(N));
    return true;
  }
  if (N != 0)
    cx3d << "MESH::Mesh::addChild: a node of type " << N->getTypeName() << " cannot be a child!" << std::endl;
  else
    cx3d << "MESH::Mesh::addChild: cannot add a NULL node!" << std::endl;
 
  return false;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
bool Mesh<MData, VData, EData, FData, readOnly>::setChild(const SFAbstractNode &N)
{
  if (dynamic_cast<Vertex<VData, EData, FData, readOnly> *>(N) != 0)
  {
    setVertex(static_cast<Vertex<VData, EData, FData, readOnly> *>(N));
    return true;
  }
  if (N != 0)
    cx3d << "MESH::Mesh::setChild: a node of type " << N->getTypeName() << " cannot be a child!" << std::endl;
  else
    cx3d << "MESH::Mesh::setChild: cannot add a NULL node!" << std::endl;
  
  return false;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
bool Mesh<MData, VData, EData, FData, readOnly>::removeChild(const SFAbstractNode &N)
{
  if (vertex == N)
  {
    setVertex(0);
    return true;
  }
  
  return false;
}    

template<class MData, class VData, class EData, class FData, bool readOnly>
SFEdge<EData, FData, VData, readOnly> *Mesh<MData, VData, EData, FData, readOnly>::getEdge(SFVertex *from, SFVertex *to)
{
  if ((from == 0) || (to == 0))
    return 0;
  
  const typename SFVertex::MFEdge &from_edges = from->getEdges();
  
  for (typename SFVertex::MFEdge::const_iterator it = from_edges.begin(); it != from_edges.end(); ++it)
  {
    if ((*it)->getToVertex() == to) 
      return (*it);
  }
      
  return 0;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const typename Mesh<MData, VData, EData, FData, readOnly>::MFVertex &Mesh<MData, VData, EData, FData, readOnly>::getVertices() const 
{
  if (vertex != 0)
    return static_cast<Vertex<VData, EData, FData, readOnly> *>(vertex)->getVertices();
  
  cx3d << "MESH::Mesh::getVertices: cannot return vertices because Vertex is NULL!" << std::endl;  
  static typename Mesh<MData, VData, EData, FData, readOnly>::MFVertex mv;
  return mv;
}

template<class MData, class VData, class EData, class FData, bool readOnly> 
const typename Mesh<MData, VData, EData, FData, readOnly>::MFEdge &Mesh<MData, VData, EData, FData, readOnly>::getEdges() const 
{
  return _edges;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const typename Mesh<MData, VData, EData, FData, readOnly>::MFFace &Mesh<MData, VData, EData, FData, readOnly>::getFaces() const 
{
  return _faces;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
MData &Mesh<MData, VData, EData, FData, readOnly>::data() 
{
  return _data;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const MData &Mesh<MData, VData, EData, FData, readOnly>::data() const 
{
  return _data;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
template<class F>  
F &Mesh<MData, VData, EData, FData, readOnly>::getData()
{
  return _data.template get<F, false>();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
template<class F>  
F &Mesh<MData, VData, EData, FData, readOnly>::ogetData()
{
  return _data.template get<F, true>();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
SFFace<FData, VData, EData, readOnly> *Mesh<MData, VData, EData, FData, readOnly>::createFace(const MFInt32 &coordIndex, bool ccw) 
{
  return createFace(_faces.size(), coordIndex, ccw);
}

template<class MData, class VData, class EData, class FData, bool readOnly>  
SFFace<FData, VData, EData, readOnly> *Mesh<MData, VData, EData, FData, readOnly>::createFace(unsigned int i, const MFInt32 &coordIndex, bool ccw) 
{
  // Creating the Face.
  SFFace *F = new SFFace(i);
  _faces.insert(std::pair<unsigned int, SFFace *>(i, F));
  
  if (ccw)
  {
    // Creating the Edges.
    MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    SFVertex *A, *B;
    SFEdge *E;
  
    const MFVertex &mv = getVertices();
    
    if ((*itCoordIndex < 0) || (static_cast<unsigned int>(*itCoordIndex) >= mv.size())) return 0;
    SFVertex *start = mv[*itCoordIndex]; 
  
    A = start;
    ++itCoordIndex;
  
    while (itCoordIndex != coordIndex.end())
    {
      if ((*itCoordIndex < 0) || (static_cast<unsigned int>(*itCoordIndex) >= mv.size())) return 0;
      B = mv[*itCoordIndex];
      ++itCoordIndex;
    
      E = getEdge(A, B);
      if (E == 0)
        E = createEdge(A, B);
      
      E->addLeftFace(F);    
      E->getFromVertex()->addFace(F);    
      F->addEdge(E);
    
      A = B;
    }
    B  = start;
    E = getEdge(A, B);
    if (E == 0)
      E = createEdge(A, B);
  
    E->addLeftFace(F);    
    E->getFromVertex()->addFace(F);
    F->addEdge(E);
  }
  else
  {
    // Creating the Edges.
    MFInt32::const_reverse_iterator itCoordIndex = coordIndex.rbegin();
    SFVertex *A, *B;
    SFEdge *E;
  
    const MFVertex &mv = getVertices();
    
    if ((*itCoordIndex < 0) || (static_cast<unsigned int>(*itCoordIndex) >= mv.size())) return 0;
    SFVertex *start = mv[*itCoordIndex]; 
  
    A = start;
    ++itCoordIndex;
  
    while (itCoordIndex != coordIndex.rend())
    {
      if ((*itCoordIndex < 0) || (static_cast<unsigned int>(*itCoordIndex) >= mv.size())) return 0;
      B = mv[*itCoordIndex];
      ++itCoordIndex;
    
      E = getEdge(A, B);
      if (E == 0)
        E = createEdge(A, B);
      
      E->addLeftFace(F);    
      E->getFromVertex()->addFace(F);    
      F->addEdge(E);
    
      A = B;
    }
    B = start;
    E = getEdge(A, B);
    if (E == 0)
      E = createEdge(A, B);
  
    E->addLeftFace(F);    
    E->getFromVertex()->addFace(F);
    F->addEdge(E);
  }
          
  return F;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
Mesh<MData, VData, EData, FData, readOnly>::Mesh(const Mesh &N)
: X3DMeshNode(N), _edges(N._edges), _faces(N._faces), _data(N._data)
{
  addChild(N.vertex);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
SFEdge<EData, FData, VData, readOnly> *Mesh<MData, VData, EData, FData, readOnly>::createEdge(SFVertex *from, SFVertex *to) 
{
  if ((from == 0) || (to == 0))
    return 0;
 
  SFEdgeContent<EData, FData, VData, readOnly> *content = new SFEdgeContent<EData, FData, VData, readOnly>(from, to);
  SFEdge *E1 = new SFDirectEdge<EData, FData, VData, readOnly>(content);
  SFEdge *E2 = new SFIndirectEdge<EData, FData, VData, readOnly>(content);
  
  E1->setSymetric(E2);
  E2->setSymetric(E1);
     
  _edges.insert(E1);
  
  // updating content of A and B.
  // to complete... to respect order of edges.
  from->addEdge(E1);
  to->addEdge(E2);
  
  return E1;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void Mesh<MData, VData, EData, FData, readOnly>::removeFace(SFFace *face)
{
  const typename SFFace::MFEdge &me = face->getEdges();
  for (typename SFFace::MFEdge::const_iterator e = me.begin(); e != me.end(); ++e)
  {
    (*e)->removeLeftFace(face);
//    std::cout << **e << " removes " << *face << std::endl;
    (*e)->getFromVertex()->removeFace(face);
//    std::cout << *(*e)->getFromVertex() << " removes " << *face << std::endl;
  }

  for (typename SFFace::MFEdge::const_iterator e = me.begin(); e != me.end(); ++e)
  {
    if ((*e)->getRightFaces().size() == 0 && (*e)->getLeftFaces().size() == 0)
    {
      (*e)->getFromVertex()->removeEdge(*e);
//      std::cout << *(*e)->getFromVertex() << " removes " << **e << std::endl;
      delete (*e)->_edgeContent;
      delete (*e)->getSymetric();
      delete (*e);
    }  
  }

  delete face;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void Mesh<MData, VData, EData, FData, readOnly>::removeFace(unsigned int i)
{
  typename MFVertex::iterator it = _faces.find(i);
  removeFace(*it);
}

}
}
