////////////////////////////////////////////////////////////////////////////////
//                            MESH_MeshData.inl                               //
////////////////////////////////////////////////////////////////////////////////

namespace X3DTK {
namespace MESH {


template<class FData, class VData, class EData, bool readOnly>
void VertexNormalData::setNormalOfFace(const SFVec3f &normal, SFFace<FData, VData, EData, readOnly> *face)
{
  // iterating the normals to find whether another one is identical.
  unsigned short i = 0;
  for (MFVec3f::iterator it = _normalArray.begin(); it != _normalArray.end(); ++it)
  {
    // found an identical normal.
    if (*it == normal)
    {
      _normalMap[static_cast<void *>(face)] = i;
      return;
    }
    ++i;
  }  
  // did not found an identical normal.
  _normalMap[static_cast<void *>(face)] = _normalArray.size();
  _normalArray.push_back(normal);
}

template<class FData, class VData, class EData, bool readOnly>
const SFVec3f &VertexNormalData::getNormalOfFace(SFFace<FData, VData, EData, readOnly> *face) const
{
  std::map<void *, unsigned short>::const_iterator it = _normalMap.find(static_cast<void *>(face));
  if (it != _normalMap.end())
    return _normalArray[(*it).second];
    
  return SFVec3f::null;   
}

template<class FData, class VData, class EData, bool readOnly>
void VertexColorData::setColorOfFace(const SFColorRGBA &color, SFFace<FData, VData, EData, readOnly> *face)
{
  unsigned short i = 0;
  for (MFColorRGBA::iterator it = _colorArray.begin(); it != _colorArray.end(); ++it)
  {
    if (*it == color)
    {
      _colorMap[static_cast<void *>(face)] = i;
      return;
    }
    ++i;
  }  
  _colorMap[static_cast<void *>(face)] = _colorArray.size();
  _colorArray.push_back(color);  
}

template<class FData, class VData, class EData, bool readOnly>
const SFColorRGBA &VertexColorData::getColorOfFace(SFFace<FData, VData, EData, readOnly> *face) const
{
  std::map<void *, unsigned short>::const_iterator it = _colorMap.find(static_cast<void *>(face));
  if (it != _colorMap.end())
    return _colorArray[(*it).second];
    
  return SFColorRGBA::null;  
}

template<class FData, class VData, class EData, bool readOnly>
void VertexTexCoordData::setTexCoordOfFace(const SFVec2f &texCoord, SFFace<FData, VData, EData, readOnly> *face)
{
  unsigned short i = 0;
  for (MFVec2f::iterator it = _texCoordArray.begin(); it != _texCoordArray.end(); ++it)
  {
    if (*it == texCoord)
    {
      _texCoordMap[static_cast<void *>(face)] = i;
      return;
    }
    ++i;
  }  
  _texCoordMap[static_cast<void *>(face)] = _texCoordArray.size();
  _texCoordArray.push_back(texCoord); 
}

template<class FData, class VData, class EData, bool readOnly>
const SFVec2f &VertexTexCoordData::getTexCoordOfFace(SFFace<FData, VData, EData, readOnly> *face) const
{
  std::map<void *, unsigned short>::const_iterator it = _texCoordMap.find(static_cast<void *>(face));
  if (it != _texCoordMap.end())
    return _texCoordArray[(*it).second];
    
  return SFVec2f::null;
}

}
}
