namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
NormalsDrawerCoreVisitor<MData, VData, EData, FData, readOnly>::NormalsDrawerCoreVisitor()
{
  // Enter and leave functions.
  defineEnterFunction(&NormalsDrawerCoreVisitor::enterTransform);
  defineEnterFunction(&NormalsDrawerCoreVisitor::enterMesh);
  defineLeaveFunction(&NormalsDrawerCoreVisitor::leaveTransform);

  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<NormalsDrawerStateVariables<MData, VData, EData, FData, readOnly> >();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void NormalsDrawerCoreVisitor<MData, VData, EData, FData, readOnly>::enterMesh(Mesh<MData, VData, EData, FData, readOnly> *M) const
{
  const typename Mesh<MData, VData, EData, FData, readOnly>::MFFace &faces = M->getFaces();
 
  float amplitude = stateVariables->getAmplitude();

  glColor3f(0.2f, 0.2f, 0.9f);

  glBegin(GL_LINES);
  for (typename Mesh<MData, VData, EData, FData, readOnly>::MFFace::const_iterator f = faces.begin(), end = faces.end() ; f != end; ++f)
  {
    const typename SFFace<FData, VData, EData, readOnly>::MFEdge &edges = (*f)->getEdges();
  
    if (!M->template ogetData<MESH::MeshNormalData>().getNormalPerVertex())
    {
      SFPoint3f center;
    
      // computing the center of the face.
      typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator e = edges.begin();
    
      if (e != edges.end())
        center = (*edges.begin())->getFromVertex()->template ogetData<MESH::VertexPointData>().getPoint();
      else
        break;
    
      ++e;  
      while (e != edges.end())
      {
        center += (*e)->getFromVertex()->template ogetData<MESH::VertexPointData>().getPoint();
        ++e;
      }
    
      float n = 1.0f/static_cast<float>(edges.size());
      center = n*center;
    
      SFPoint3f normal = center + amplitude*((*f)->template ogetData<MESH::FaceNormalData>().getNormal());
    
      // drawing.
      glVertex3fv(center.f_data());
      glVertex3fv(normal.f_data());  
    }
    else
    {
      for (typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator e = edges.begin(), end = edges.end() ; e != end; ++e)
      {
        SFPoint3f center = (*e)->getFromVertex()->template ogetData<MESH::VertexPointData>().getPoint();
        SFPoint3f normal = center + amplitude*((*e)->getFromVertex()->template ogetData<MESH::VertexNormalData>().getNormalOfFace(*f));
      
        // drawing.
        glVertex3fv(center.f_data());
        glVertex3fv(normal.f_data()); 
      }
    }  
  }
  glEnd();
}
template<class MData, class VData, class EData, class FData, bool readOnly>
void NormalsDrawerCoreVisitor<MData, VData, EData, FData, readOnly>::enterTransform(Transform *T) const
{
  glMatrixMode(GL_MODELVIEW);
  stateVariables->pushMatrix(T->getTransform());
  glPushMatrix();
  glMultMatrixf(T->getTransform().toFloat16());
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void NormalsDrawerCoreVisitor<MData, VData, EData, FData, readOnly>::leaveTransform(Transform *) const
{
  glMatrixMode(GL_MODELVIEW);
  stateVariables->popMatrix();
  glPopMatrix();
}

}
}
