namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
NormalsFlipperCoreVisitor<MData, VData, EData, FData, readOnly>::NormalsFlipperCoreVisitor()
{
  // Enter and leave functions.
  defineEnterFunction(&NormalsFlipperCoreVisitor::enterVertex);
  defineWalkOnFunction(&NormalsFlipperCoreVisitor::walkOnShape);
    
  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<NormalsFlipperStateVariables<MData, VData, EData, FData, readOnly> >();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
bool NormalsFlipperCoreVisitor<MData, VData, EData, FData, readOnly>::walkOnShape(Shape *, SFAbstractNode child) const
{
  MFNode selectedNodes = stateVariables->getSelectedNodes();
  return (selectedNodes.empty() || (std::find(selectedNodes.begin(), selectedNodes.end(), child) != selectedNodes.end()));
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void NormalsFlipperCoreVisitor<MData, VData, EData, FData, readOnly>::enterVertex(Vertex<VData, EData, FData, readOnly> *V) const
{
  if (!stateVariables->getNode(V))
  { 
    // computing the list of Mesh parents of V.
    MFAbstractNode parentList = V->getParentList();
    std::list<Mesh<MData, VData, EData, FData, readOnly> *> meshList;
    for (MFAbstractNode::const_iterator it = parentList.begin(); it != parentList.end(); ++it)
    {
      if (dynamic_cast<Mesh<MData, VData, EData, FData, readOnly> *>(*it) != 0)
        meshList.push_back(static_cast<Mesh<MData, VData, EData, FData, readOnly> *>(*it));
    }
    
    // defining structures.
    typename std::vector<SFFace<FData, VData, EData, readOnly> *> stack;
    unsigned int stackSize = 0;
    for (typename std::list<Mesh<MData, VData, EData, FData, readOnly> *>::const_iterator it = meshList.begin(); it != meshList.end(); ++it)
      stackSize += (*it)->getFaces().size();
    stack.reserve(stackSize);
    
    typename std::list<Mesh<MData, VData, EData, FData, readOnly> *>::const_iterator itMesh = meshList.begin();
    if (itMesh == meshList.end()) return;
    
    typename Mesh<MData, VData, EData, FData, readOnly>::MFFace::const_iterator itFace = (*itMesh)->getFaces().begin();
    
    while (itMesh != meshList.end())
    {
      std::map<SFFace<FData, VData, EData, readOnly> *, bool> faceMap;
      stack.push_back(*itFace);
      typename std::vector<SFFace<FData, VData, EData, readOnly> *>::const_iterator f = stack.end();
      --f;
      faceMap[*f] = true;
    
//    std::cout << "face = " << **f << std::endl;    
//    std::cout << "fm = 1" << std::endl;
    // initializing the algorithm by the first face. Precondition all faces can be accessed
    // by neighboorhood of the face.
    
      while (f != stack.end())
      {
        bool current = faceMap[*f];
//        std::cout << &(*f) << " -> " << *f << std::endl;
            
/*      if ((unsigned int)(*f) == 0x41)
      {
        for (typename Mesh<MData, VData, EData, FData, readOnly>::MFFace::const_iterator it = stack.begin(); it != stack.end(); ++it)
          std::cout << &(*it) << " -> " << *it << ", ";
        std::cout << std::endl;  
      }*/

        // searching the list of faces around f.
        const typename SFFace<FData, VData, EData, readOnly>::MFEdge &me = (*f)->getEdges();
        for (typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator e = me.begin(); e != me.end(); ++e)
        {
//        std::cout << "edge = " << **e << std::endl;
          const typename SFEdge<EData, FData, VData, readOnly>::MFFace &mlf = (*e)->getLeftFaces();
          const typename SFEdge<EData, FData, VData, readOnly>::MFFace &mrf = (*e)->getRightFaces();
        
          for (typename SFEdge<EData, FData, VData, readOnly>::MFFace::const_iterator nf = mlf.begin(); nf != mlf.end(); ++nf)
          {
//          std::cout << "l face = " << **nf << std::endl;
            if (faceMap.find(*nf) == faceMap.end())
            {
              faceMap[*nf] = !current;
                
//              std::cout << "fm = " << faceMap[*nf] << std::endl;            
              stack.push_back(*nf);
            }
          }
        
          for (typename SFEdge<EData, FData, VData, readOnly>::MFFace::const_iterator nf = mrf.begin(); nf != mrf.end(); ++nf)
          {
//            std::cout << "r face = " << **nf << std::endl;
            if (faceMap.find(*nf) == faceMap.end())
            {
              faceMap[*nf] = current;
              
//            std::cout << "fm = " << faceMap[*nf] << std::endl;          
            
              stack.push_back(*nf);
            }
          }
        }
        ++f;
      }
    
      // Counting the number of direct faces against indirect faces.
      unsigned int direct = 0;
      for (typename std::map<SFFace<FData, VData, EData, readOnly> *, bool>::const_iterator it = faceMap.begin(); it != faceMap.end(); ++it)
      {
        if ((*it).second)
          ++direct;
      }
    
      unsigned int nfaces = 0;
      if (direct < faceMap.size() - direct)
        nfaces = direct;
      else
        nfaces = faceMap.size() - direct;
        
      std::cout << "reversing " << nfaces << "/" << faceMap.size() << " faces" << std::endl;
    
      for (typename std::map<SFFace<FData, VData, EData, readOnly> *, bool>::const_iterator pf = faceMap.begin(); pf != faceMap.end(); ++pf)
        if ((direct > (faceMap.size() >> 1)) != ((*pf).second)) 
        {
//          std::cout << "reversing " << *(*it).first << std::endl;
          SFFace<FData, VData, EData, readOnly> *f = (*pf).first;
          f->reverse();
          // reversing the normals of the face.
          if (FData::template find<MESH::FaceNormalData>())
          {
            const SFVec3f &n = f->template ogetData<MESH::FaceNormalData>().getNormal();
            f->template ogetData<MESH::FaceNormalData>().setNormal(-n);
          }  
  
          if (VData::template find<MESH::VertexNormalData>())
          {
            const typename SFFace<FData, VData, EData, readOnly>::MFEdge &me = f->getEdges();
            for (typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator e = me.begin(); e != me.end(); ++e)
            {
              SFVertex<VData, EData, FData, readOnly> *v = (*e)->getFromVertex();
              const SFVec3f &n = v->template ogetData<MESH::VertexNormalData>().getNormalOfFace(f);
              v->template ogetData<MESH::VertexNormalData>().setNormalOfFace(-n, f);
            }
          }  
        }
      
      // finding the next face.
      while ((itMesh != meshList.end()) && (std::find(stack.begin(), stack.end(), *itFace) != stack.end()))
      {
        ++itFace;
        if (itFace == (*itMesh)->getFaces().end())
        {
          ++itMesh;
          if (itMesh != meshList.end())
            itFace = (*itMesh)->getFaces().begin();
        }    
      }  
    }
    stateVariables->addNode(V);  
  }
}

}
}  
