namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
NormalsInverterCoreVisitor<MData, VData, EData, FData, readOnly>::NormalsInverterCoreVisitor()
{
  // Enter and leave functions.
  defineEnterFunction(&NormalsInverterCoreVisitor::enterMesh);
  defineWalkOnFunction(&NormalsInverterCoreVisitor::walkOnShape);
  
  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<NormalsInverterStateVariables<MData, VData, EData, FData, readOnly> >();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
bool NormalsInverterCoreVisitor<MData, VData, EData, FData, readOnly>::walkOnShape(Shape *, SFAbstractNode child) const
{
  MFNode selectedNodes = stateVariables->getSelectedNodes();
  return (selectedNodes.empty() || (std::find(selectedNodes.begin(), selectedNodes.end(), child) != selectedNodes.end()));
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void NormalsInverterCoreVisitor<MData, VData, EData, FData, readOnly>::enterMesh(Mesh<MData, VData, EData, FData, readOnly> *M) const
{
  if (!stateVariables->getNode(M))
  { 
    const typename Mesh<MData, VData, EData, FData, readOnly>::MFFace &mf = M->getFaces();
    for (typename Mesh<MData, VData, EData, FData, readOnly>::MFFace::const_iterator pf = mf.begin(); pf != mf.end(); ++pf)
    {
      typename Mesh<MData, VData, EData, FData, readOnly>::SFFace *f = *pf;
      f->reverse();
      // reversing the normals of the face.
      if (FData::template find<MESH::FaceNormalData>())
      {
        const SFVec3f &n = f->template ogetData<MESH::FaceNormalData>().getNormal();
        f->template ogetData<MESH::FaceNormalData>().setNormal(-n);
      }  
  
      if (VData::template find<MESH::VertexNormalData>())
      {
        const typename SFFace<FData, VData, EData, readOnly>::MFEdge &me = f->getEdges();
        for (typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator e = me.begin(); e != me.end(); ++e)
        {
          SFVertex<VData, EData, FData, readOnly> *v = (*e)->getFromVertex();
          const SFVec3f &n = v->template ogetData<MESH::VertexNormalData>().getNormalOfFace(f);
          v->template ogetData<MESH::VertexNormalData>().setNormalOfFace(-n, f);
        }
      }  
    }  
    
    stateVariables->addNode(M);  
  }
}

}
}  
