/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_NORMALSUPDATER_H
#define X3DTK_MESH_NORMALSUPDATER_H

#include "MESH_NormalsUpdaterStateVariables.h"
#include "MESH_SceneGraphTypes.h"
#include "X3DMultiPassProcessor.h"
#include "MESH_NormalsUpdaterFacesCoreVisitor.h"
#include "MESH_NormalsUpdaterVerticesCoreVisitor.h"
#include "MESH_DFSGraphTraversal.h"
#include "MESH_X3DNode.h"
#include "MESH_MeshData.h"

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/normalsupdater.h></b> providing a processor that 
 *  updates the normals of a MESH scene graph.
 *
 *  \ingroup MESH
 */

template<class MData, class VData, class EData, class FData, bool readOnly>
class NormalsUpdater : public X3DMultiPassProcessor
{
public:
  /// Constructor.
  NormalsUpdater();
  /// Destructor.
  virtual ~NormalsUpdater();
  
  /// Updates the normals.
  void updateNormals(X3DNode *N);
  /// Sets the selected nodes to update. If empty, then all the graph is updated.
  void setSelectedNodes(const MFNode &selectedNodes);

protected:
  NormalsUpdaterStateVariables<MData, VData, EData, FData, readOnly> *stateVariables;
};

/// Default NormalsUpdater
typedef NormalsUpdater<MeshData, VertexData, EdgeData, FaceData, true> DefNormalsUpdater;

}
}

#include "MESH_NormalsUpdater.inl"

#endif

