namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
NormalsUpdater<MData, VData, EData, FData, readOnly>::NormalsUpdater()
: X3DMultiPassProcessor()
{
  stateVariables = GraphTraversal::getInstanceOf<NormalsUpdaterStateVariables<MData, VData, EData, FData, readOnly> >();
  
  graphTraversalMap["Faces"] = new DFSGraphTraversal();
  graphTraversalMap["Vertices"] = new DFSGraphTraversal();

  graphTraversalMap["Faces"]->setComponentVisitor(new NormalsUpdaterFacesCoreVisitor<MData, VData, EData, FData, readOnly>());
  graphTraversalMap["Vertices"]->setComponentVisitor(new NormalsUpdaterVerticesCoreVisitor<MData, VData, EData, FData, readOnly>());
}

template<class MData, class VData, class EData, class FData, bool readOnly>
NormalsUpdater<MData, VData, EData, FData, readOnly>::~NormalsUpdater()
{
  for (MFGraphTraversal::iterator it = graphTraversalMap.begin(); it != graphTraversalMap.end(); ++it)
    delete (*it).second; 
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void NormalsUpdater<MData, VData, EData, FData, readOnly>::updateNormals(X3DNode *N)
{
  if (VData::template find<MESH::VertexNormalData>() 
   && FData::template find<MESH::FaceNormalData>())
  {
    graphTraversalMap["Faces"]->traverse(N);
  }
  
  if (VData::template find<MESH::VertexNormalData>() 
   && FData::template find<MESH::FaceNormalData>())
  { 
    graphTraversalMap["Vertices"]->traverse(N);
  }  
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void NormalsUpdater<MData, VData, EData, FData, readOnly>::setSelectedNodes(const MFNode &selectedNodes)
{
  stateVariables->setSelectedNodes(selectedNodes);
}

}
}
