namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
NormalsUpdaterFacesCoreVisitor<MData, VData, EData, FData, readOnly>::NormalsUpdaterFacesCoreVisitor()
{
  defineEnterFunction(&NormalsUpdaterFacesCoreVisitor::enterMesh);
  defineWalkOnFunction(&NormalsUpdaterFacesCoreVisitor::walkOnShape);
  
  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<NormalsUpdaterStateVariables<MData, VData, EData, FData, readOnly> >();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
bool NormalsUpdaterFacesCoreVisitor<MData, VData, EData, FData, readOnly>::walkOnShape(Shape *, SFAbstractNode child) const
{
  MFNode selectedNodes = stateVariables->getSelectedNodes();
  return (selectedNodes.empty() || (std::find(selectedNodes.begin(), selectedNodes.end(), child) != selectedNodes.end()));
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void NormalsUpdaterFacesCoreVisitor<MData, VData, EData, FData, readOnly>::enterMesh(Mesh<MData, VData, EData, FData, readOnly> *M) const
{
  if (VData::template find<MESH::VertexPointData>() 
   && FData::template find<MESH::FaceNormalData>())
  { 
    // Compute all faces normals 
    const typename Mesh<MData, VData, EData, FData, readOnly>::MFFace &faces = M->getFaces();
    for (typename Mesh<MData, VData, EData, FData, readOnly>::MFFace::const_iterator f = faces.begin(), end = faces.end() ; f != end; ++f)
    {
      SFVec3f normal;
      const typename SFFace<FData, VData, EData, readOnly>::MFEdge &edges = (*f)->getEdges();
      typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator e = edges.begin();
      // Choose a pivot point
      const SFPoint3f &firstPoint = (*e)->getFromVertex()->template ogetData<MESH::VertexPointData>().getPoint();

      // Initialize secondPoint from second edge
      ++e;
      SFPoint3f secondPoint = (*e)->getFromVertex()->template ogetData<MESH::VertexPointData>().getPoint();

      // Start loop at third edge
      ++e;
      for (typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator end = edges.end() ; e != end; ++e)
      {
        SFPoint3f thirdPoint = (*e)->getFromVertex()->template ogetData<MESH::VertexPointData>().getPoint();
        normal = normal + crossprod(secondPoint - firstPoint, thirdPoint - firstPoint);
        secondPoint = thirdPoint;
      }

      // Update face normal (if face does not already has a normal)
//      if ((*f)->template ogetData<MESH::FaceNormalData>().getNormal().norm() < 1E-4)
        (*f)->template ogetData<MESH::FaceNormalData>().setNormal(normal.normalize());
    }
  }  
}

}
}
