namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
NormalsUpdaterVerticesCoreVisitor<MData, VData, EData, FData, readOnly>::NormalsUpdaterVerticesCoreVisitor()
{
  defineEnterFunction(&NormalsUpdaterVerticesCoreVisitor::enterMesh);
  defineWalkOnFunction(&NormalsUpdaterVerticesCoreVisitor::walkOnShape);
    
  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<NormalsUpdaterStateVariables<MData, VData, EData, FData, readOnly> >();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
bool NormalsUpdaterVerticesCoreVisitor<MData, VData, EData, FData, readOnly>::walkOnShape(Shape *, SFAbstractNode child) const
{
  MFNode selectedNodes = stateVariables->getSelectedNodes();
  return (selectedNodes.empty() || (std::find(selectedNodes.begin(), selectedNodes.end(), child) != selectedNodes.end()));
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void NormalsUpdaterVerticesCoreVisitor<MData, VData, EData, FData, readOnly>::enterMesh(Mesh<MData, VData, EData, FData, readOnly> *M) const
{
  if (VData::template find<MESH::VertexNormalData>() 
   && FData::template find<MESH::FaceNormalData>())
  { 
    const typename Mesh<MData, VData, EData, FData, readOnly>::MFVertex &vertices = M->getVertices();
    for (typename Mesh<MData, VData, EData, FData, readOnly>::MFVertex::const_iterator v = vertices.begin(), end = vertices.end() ; v != end; ++v)
    {
      SFVec3f normal;

      // Compute point normal as an area weighted average of faces' normals
      const typename SFVertex<VData, EData, FData, readOnly>::MFFace &faces = (*v)->getFaces();

      for (typename SFVertex<VData, EData, FData, readOnly>::MFFace::const_iterator f = faces.begin(), end = faces.end() ; f != end; ++f)
        normal = normal + (*f)->template ogetData<MESH::FaceNormalData>().getNormal();

      normal.normalize();

      // Store computed normal in VertexFData
      for (typename SFVertex<VData, EData, FData, readOnly>::MFFace::const_iterator f = faces.begin(), end = faces.end() ; f != end; ++f)
      {  
//        if ((*v)->template ogetData<MESH::VertexNormalData>().getNormalOfFace(*f).norm() < 1E-4)
          (*v)->template ogetData<MESH::VertexNormalData>().setNormalOfFace(normal, *f);
      } 
    }
  }
}

}
}
