/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_SFEDGE_H
#define X3DTK_MESH_SFEDGE_H

#include "MESH_SceneGraphTypes.h"
#include "MESH_SFEdgeContent.h"

namespace X3DTK {
namespace MESH {

template<class VData, class EData, class FData, bool readOnly> class SFVertex;
template<class FData, class VData, class EData, bool readOnly> class SFFace;

/*! \brief Template class declared in <b><X3DTK/MESH/scenegraph.h></b> 
 *  providing an edge structure for the Mesh class.
 *
 * \ingroup MeshSceneGraph
 */

template<class EData, class FData, class VData, bool readOnly = true>
class SFEdge
{
  template<class A, class B, class C, class D, bool E> friend class Mesh;
  template<class A, class B, class C, bool D> friend class SFDirectEdge;
  template<class A, class B, class C, bool D> friend class SFIndirectEdge;
  template<class A, class B, class C, bool D> friend class SFFace;
public:
  typedef EntityMFFace<FData, VData, EData, readOnly> MFFace;
  
  /// Returns the symetric edge.
  inline SFEdge *getSymetric();
  /// Returns the Vertex from.
  virtual SFVertex<VData, EData, FData, readOnly> *getFromVertex() const = 0;
  /// Gets the Vertex to.
  virtual SFVertex<VData, EData, FData, readOnly> *getToVertex() const = 0;
  /// Gets the left faces.
  virtual const MFFace &getLeftFaces() const = 0;
  /// Gets the right faces.
  virtual const MFFace &getRightFaces() const = 0;
  /// Gets the faces.
  MFFace getFaces() const;
  /// Returns true if is a boundary.
  inline bool isBoundary() const;
  /// Returns true if is non-manifold.
  inline bool isNonManifold() const;
  /// Gets the data.
  inline EData &data();
  /// Gets the data.
  inline const EData &data() const;
  /// Gets the data F from EData which must be of type clist<tlist<T...> >.
  template<class F>  
  F &getData();
  /// Gets optionally the data F from EData which must be of type clist<tlist<T...> >.
  template<class F>  
  F &ogetData();
  
protected:    
  /// Constructor.
  SFEdge(SFEdgeContent<EData, FData, VData, readOnly> *const edgeContent);
  /// Destructor.
  virtual ~SFEdge();
  
  /// Sets the symetric edge.
  void setSymetric(SFEdge *symetric);
  /// Adds a left face.
  virtual void addLeftFace(SFFace<FData, VData, EData, readOnly> *face) = 0;
  /// Adds a right face.
  virtual void addRightFace(SFFace<FData, VData, EData, readOnly> *face) = 0;

  /// Removes a left face.
  virtual void removeLeftFace(SFFace<FData, VData, EData, readOnly> *face) = 0;
  /// Removes a right face.
  virtual void removeRightFace(SFFace<FData, VData, EData, readOnly> *face) = 0;

private:
  SFEdgeContent<EData, FData, VData, readOnly> *const _edgeContent;
  SFEdge *_symetric;
  
  /// Reverses the face. Is called by SFFace::reverse().
  void reverse(SFFace<FData, VData, EData, readOnly> *face);
};

/// operator<< overloaded for SFEdge class.
template<class EData, class FData, class VData, bool readOnly>
std::ostream& operator<<(std::ostream& o, const SFEdge<EData, FData, VData, readOnly> &e);

}
}

#include "MESH_SFEdge.inl"

#endif
