////////////////////////////////////////////////////////////////////////////////
//                            MESH_SFEdge.inl                                 //
////////////////////////////////////////////////////////////////////////////////

namespace X3DTK {
namespace MESH {

template<class EData, class FData, class VData, bool readOnly>
SFEdge<EData, FData, VData, readOnly> *SFEdge<EData, FData, VData, readOnly>::getSymetric() 
{
  return _symetric;
}

template<class EData, class FData, class VData, bool readOnly>
typename SFEdge<EData, FData, VData, readOnly>::MFFace SFEdge<EData, FData, VData, readOnly>::getFaces() const 
{
  MFFace faces;
  const MFFace &r1faces = _edgeContent->get1Faces();
  const MFFace &r2faces = _edgeContent->get2Faces();
  
  //faces.reserve(r1faces.size() + r2faces.size() + 1);
  for (typename MFFace::const_iterator it = r1faces.begin(); it != r1faces.end(); ++it)
    faces.push_back(*it);

  for (typename MFFace::const_iterator it = r2faces.begin(); it != r2faces.end(); ++it)
    faces.push_back(*it);

  return faces;
}

template<class EData, class FData, class VData, bool readOnly>
bool SFEdge<EData, FData, VData, readOnly>::isBoundary() const 
{
  return _edgeContent->isBoundary();
}

template<class EData, class FData, class VData, bool readOnly>
bool SFEdge<EData, FData, VData, readOnly>::isNonManifold() const 
{
  return _edgeContent->isNonManifold();
}

template<class EData, class FData, class VData, bool readOnly>
EData &SFEdge<EData, FData, VData, readOnly>::data() 
{
  return _edgeContent->data();
}

template<class EData, class FData, class VData, bool readOnly>
const EData &SFEdge<EData, FData, VData, readOnly>::data() const 
{
  return _edgeContent->data();
}

template<class EData, class FData, class VData, bool readOnly>
template<class F>  
F &SFEdge<EData, FData, VData, readOnly>::getData()
{
  return _data.template get<F, false>();
}

template<class EData, class FData, class VData, bool readOnly>
template<class F>  
F &SFEdge<EData, FData, VData, readOnly>::ogetData()
{
  return _data.template get<F, true>();
}

template<class EData, class FData, class VData, bool readOnly>
SFEdge<EData, FData, VData, readOnly>::SFEdge(SFEdgeContent<EData, FData, VData, readOnly> *const edgeContent)
: _edgeContent(edgeContent), _symetric(0)
{
}

template<class EData, class FData, class VData, bool readOnly>
SFEdge<EData, FData, VData, readOnly>::~SFEdge() 
{
}

template<class EData, class FData, class VData, bool readOnly>
void SFEdge<EData, FData, VData, readOnly>::setSymetric(SFEdge *symetric) 
{
  _symetric = symetric;
}

template<class EData, class FData, class VData, bool readOnly>
void SFEdge<EData, FData, VData, readOnly>::reverse(SFFace<FData, VData, EData, readOnly> *face)
{
  _edgeContent->reverse(face);
}

template<class EData, class FData, class VData, bool readOnly>
std::ostream& operator<<(std::ostream& o, const SFEdge<EData, FData, VData, readOnly> &e)
{
  o << "< " << e.getFromVertex()->getIndex() << ", " << e.getToVertex()->getIndex() << " >";
  return o;
}

}
}
