/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_SFEDGECONTENT_H
#define X3DTK_MESH_SFEDGECONTENT_H

#include "MESH_SceneGraphTypes.h"

#include <vector>
#include <algorithm>

namespace X3DTK {
namespace MESH {

template<class VData, class EData, class FData, bool readOnly> class SFVertex;
template<class FData, class VData, class EData, bool readOnly> class SFFace;

/*! \brief Template class declared in <b><X3DTK/MESH/scenegraph.h></b> 
 *  providing the shared content of an SFEdge structure.
 *
 * \ingroup MeshSceneGraph
 */

template<class EData, class FData, class VData, bool readOnly = true>
class SFEdgeContent
{
  template<class A, class B, class C, class D, bool E> friend class Mesh;
  template<class A, class B, class C, bool D> friend class SFEdge;
  template<class A, class B, class C, bool D> friend class SFDirectEdge;
  template<class A, class B, class C, bool D> friend class SFIndirectEdge;
public:
  typedef EntityMFFace<FData, VData, EData, readOnly> MFFace;
 
  /// Gets the Vertex from.
  inline SFVertex<VData, EData, FData, readOnly> *getFromVertex() const;
  /// Gets the Vertex to.
  inline SFVertex<VData, EData, FData, readOnly> *getToVertex() const;
  /// Gets the 1-faces.
  inline const MFFace &get1Faces() const;
  /// Gets the 2-faces.
  inline const MFFace &get2Faces() const;
  /// Returns true if is a boundary.
  inline bool isBoundary() const;
  /// Returns true if is non-manifold.
  inline bool isNonManifold() const;
  /// Gets the data.
  inline EData &data();
  /// Gets the data.
  inline const EData &data() const;

private:
  SFVertex<VData, EData, FData, readOnly> *const _from;
  SFVertex<VData, EData, FData, readOnly> *const _to;
  MFFace _r1Faces;
  MFFace _r2Faces;
  EData _data;
  
  /// Constructor.
  SFEdgeContent(SFVertex<VData, EData, FData, readOnly> * from, SFVertex<VData, EData, FData, readOnly> *const to);
  /// Destructor.
  ~SFEdgeContent();
    
  /// Adds a 1-face.
  void add1Face(SFFace<FData, VData, EData, readOnly> *face);
  /// Adds a 2-face.
  void add2Face(SFFace<FData, VData, EData, readOnly> *face);
  
  /// Removes a 1-face.
  void remove1Face(SFFace<FData, VData, EData, readOnly> *face);
  /// Removes a 2-face.
  void remove2Face(SFFace<FData, VData, EData, readOnly> *face);
  
  /// Reverses the face, ie if face is a 1-face, it becomes a 2-face.
  void reverse(SFFace<FData, VData, EData, readOnly> *face);
};

}
}

#include "MESH_SFEdgeContent.inl"

#endif
