////////////////////////////////////////////////////////////////////////////////
//                            MESH_SFEdgeContent.inl                          //
////////////////////////////////////////////////////////////////////////////////

namespace X3DTK {
namespace MESH {

template<class EData, class FData, class VData, bool readOnly>
SFVertex<VData, EData, FData, readOnly> *SFEdgeContent<EData, FData, VData, readOnly>::getFromVertex() const 
{
  return _from;
}

template<class EData, class FData, class VData, bool readOnly>
SFVertex<VData, EData, FData, readOnly> *SFEdgeContent<EData, FData, VData, readOnly>::getToVertex() const 
{
  return _to;
}

template<class EData, class FData, class VData, bool readOnly>
const typename SFEdgeContent<EData, FData, VData, readOnly>::MFFace &SFEdgeContent<EData, FData, VData, readOnly>::get1Faces() const 
{
  return _r1Faces;
}

template<class EData, class FData, class VData, bool readOnly>
const typename SFEdgeContent<EData, FData, VData, readOnly>::MFFace &SFEdgeContent<EData, FData, VData, readOnly>::get2Faces() const 
{
  return _r2Faces;
}

template<class EData, class FData, class VData, bool readOnly>
bool SFEdgeContent<EData, FData, VData, readOnly>::isBoundary() const 
{
  return (_r2Faces.size() == 0) || (_r1Faces.size() == 0);
}

template<class EData, class FData, class VData, bool readOnly>
bool SFEdgeContent<EData, FData, VData, readOnly>::isNonManifold() const 
{
  return (_r2Faces.size() > 1) || (_r1Faces.size() > 1);
}

template<class EData, class FData, class VData, bool readOnly>
EData &SFEdgeContent<EData, FData, VData, readOnly>::data() 
{
  return _data;
}

template<class EData, class FData, class VData, bool readOnly>
const EData &SFEdgeContent<EData, FData, VData, readOnly>::data() const 
{
  return _data;
}

template<class EData, class FData, class VData, bool readOnly>
SFEdgeContent<EData, FData, VData, readOnly>::SFEdgeContent(SFVertex<VData, EData, FData, readOnly> * from, SFVertex<VData, EData, FData, readOnly> *const to) 
: _from(from), _to(to)
{
}

template<class EData, class FData, class VData, bool readOnly>
SFEdgeContent<EData, FData, VData, readOnly>::~SFEdgeContent() 
{
}

template<class EData, class FData, class VData, bool readOnly>
void SFEdgeContent<EData, FData, VData, readOnly>::add1Face(SFFace<FData, VData, EData, readOnly> *face) 
{
  _r1Faces.push_back(face);
}

template<class EData, class FData, class VData, bool readOnly>
void SFEdgeContent<EData, FData, VData, readOnly>::add2Face(SFFace<FData, VData, EData, readOnly> *face) 
{
  _r2Faces.push_back(face);
}

template<class EData, class FData, class VData, bool readOnly>
void SFEdgeContent<EData, FData, VData, readOnly>::remove1Face(SFFace<FData, VData, EData, readOnly> *face) 
{
  typename MFFace::iterator it = std::find(_r1Faces.begin(), _r1Faces.end(), face);
  if (it != _r1Faces.end())
    _r1Faces.erase(it);
}

template<class EData, class FData, class VData, bool readOnly>
void SFEdgeContent<EData, FData, VData, readOnly>::remove2Face(SFFace<FData, VData, EData, readOnly> *face) 
{
  typename MFFace::iterator it = std::find(_r2Faces.begin(), _r2Faces.end(), face);
  if (it != _r2Faces.end())
    _r2Faces.erase(it);
}

template<class EData, class FData, class VData, bool readOnly>
void SFEdgeContent<EData, FData, VData, readOnly>::reverse(SFFace<FData, VData, EData, readOnly> *face)
{
//  std::cout << "r1 size before = " << _r1Faces.size() << std::endl;
//  std::cout << "r2 size before = " << _r2Faces.size() << std::endl;
  
  typename SFEdgeContent<EData, FData, VData, readOnly>::MFFace::iterator it = std::find(_r1Faces.begin(), _r1Faces.end(), face);
  if (it == _r1Faces.end())
  {
    it = std::find(_r2Faces.begin(), _r2Faces.end(), face);
    if (it != _r2Faces.end())
    {
      _r2Faces.erase(it);
      _r1Faces.push_back(face);
    }  
  }
  else
  {
    _r1Faces.erase(it);
    _r2Faces.push_back(face);    
  }  
  
//  std::cout << "r1 size after = " << _r1Faces.size() << std::endl;
//  std::cout << "r2 size after = " << _r2Faces.size() << std::endl;
}


 
}
}
