/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_SFFACE_H
#define X3DTK_MESH_SFFACE_H

#include "MESH_SceneGraphTypes.h"

#include <vector>
#include <iostream>

namespace X3DTK {
namespace MESH {

template<class EData, class FData, class VData, bool readOnly> class SFEdge;

/*! \brief Template class declared in <b><X3DTK/MESH/scenegraph.h></b> 
 *  providing a face structure for the Mesh class.
 *
 * \ingroup MeshSceneGraph
 */

template<class FData, class VData, class EData, bool readOnly = true>
class SFFace
{
  template<class A, class B, class C, class D, bool E> friend class Mesh;
public:
  typedef EntityMFEdge<EData, FData, VData, readOnly> MFEdge;
  
  /// Gets the index.
  inline unsigned int getIndex() const;
  /// Gets the edges of the faces. The order is given b the order of definition of the face.
  inline const MFEdge &getEdges() const;
  /// Gets the data.
  inline FData &data();
  /// Gets the data.
  inline const FData &data() const;
  /// Gets the data F from FData which must be of type clist<tlist<T...> >.
  template<class F>  
  F &getData();
  /// Gets optionally the data F from FData which must be of type clist<tlist<T...> >.
  template<class F>  
  F &ogetData();
  /// Reverses the face.
  void reverse();
  
private:
  const unsigned int _index;
  MFEdge _edges;
  FData _data;
  
  /// Constructor.
  SFFace(unsigned int i);
  /// Destructor.
  ~SFFace();
  
  /// Adds an edge.
  void addEdge(SFEdge<EData, FData, VData, readOnly> *edge);
};

/// operator<< overloaded for SFFace class.
template<class FData, class VData, class EData, bool readOnly>
std::ostream& operator<<(std::ostream& o, const SFFace<FData, VData, EData, readOnly> &f);

}
}

#include "MESH_SFFace.inl"

#endif
