////////////////////////////////////////////////////////////////////////////////
//                            MESH_SFFace.inl                                 //
////////////////////////////////////////////////////////////////////////////////

namespace X3DTK {
namespace MESH {

template<class FData, class VData, class EData, bool readOnly>
unsigned int SFFace<FData, VData, EData, readOnly>::getIndex() const 
{
  return _index;
}

template<class FData, class VData, class EData, bool readOnly>
const typename SFFace<FData, VData, EData, readOnly>::MFEdge &SFFace<FData, VData, EData, readOnly>::getEdges() const 
{
  return _edges;
}

template<class FData, class VData, class EData, bool readOnly>
FData &SFFace<FData, VData, EData, readOnly>::data() 
{
  return _data;
}

template<class FData, class VData, class EData, bool readOnly>
const FData &SFFace<FData, VData, EData, readOnly>::data() const 
{ 
  return _data;
}

template<class VData, class EData, class FData, bool readOnly>
template<class F>  
F &SFFace<VData, EData, FData, readOnly>::getData()
{
  return _data.template get<F, false>();
}

template<class VData, class EData, class FData, bool readOnly>
template<class F>  
F &SFFace<VData, EData, FData, readOnly>::ogetData()
{
  return _data.template get<F, true>();
}

template<class FData, class VData, class EData, bool readOnly>
SFFace<FData, VData, EData, readOnly>::SFFace(unsigned int i)
: _index(i)
{
}

template<class FData, class VData, class EData, bool readOnly>  
SFFace<FData, VData, EData, readOnly>::~SFFace() 
{
}

template<class FData, class VData, class EData, bool readOnly>  
void SFFace<FData, VData, EData, readOnly>::addEdge(SFEdge<EData, FData, VData, readOnly> *edge)
{
  _edges.push_back(edge);
}

template<class FData, class VData, class EData, bool readOnly>  
void SFFace<FData, VData, EData, readOnly>::reverse()
{
  // reversing for each edge of the face.
  for (typename MFEdge::iterator it = _edges.begin(); it != _edges.end(); ++it)
    (*it)->reverse(this);
  
  // reversing the list of edges of the face.
  MFEdge tmp;
  for (typename MFEdge::reverse_iterator it = _edges.rbegin(); it != _edges.rend(); ++it)
  {
    tmp.push_back((*it)->getSymetric());
  }
  _edges.clear();
  _edges = tmp;
}

template<class FData, class VData, class EData, bool readOnly>
std::ostream& operator<<(std::ostream& o, const SFFace<FData, VData, EData, readOnly> &f)
{
  o << "( ";
  const typename SFFace<FData, VData, EData, readOnly>::MFEdge &edges = f.getEdges();
  if (!edges.empty())
  {
    typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator itEnd = edges.end();
    --itEnd;
    for (typename SFFace<FData, VData, EData, readOnly>::MFEdge::const_iterator it = edges.begin(); it != itEnd; ++it)
      o << (*it)->getFromVertex()->getIndex() << ", ";
  
    o << (*itEnd)->getFromVertex()->getIndex();        
  }
  o << " )";  
  
  return o;
}  

}
}  
