/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_SFVERTEX_H
#define X3DTK_MESH_SFVERTEX_H

#include "MESH_SceneGraphTypes.h"
#include "TypeList.h"

#include <vector>

namespace X3DTK {
namespace MESH {

template<class EData, class FData, class VData, bool readOnly> class SFEdge;
template<class FData, class VData, class EData, bool readOnly> class SFFace;

/*! \brief Template class declared in <b><X3DTK/MESH/scenegraph.h></b> 
 *  providing a vertex structure for the Mesh class.
 *
 * \ingroup MeshSceneGraph
 */

template<class VData, class EData, class FData, bool readOnly = true>
class SFVertex
{
  template<class A, class B, class C, bool E> friend class Vertex;
  template<class A, class B, class C, class D, bool E> friend class Mesh;
public:
  typedef EntityMFFace<FData, VData, EData, readOnly> MFFace;
  typedef EntityMFEdge<EData, FData, VData, readOnly> MFEdge;
  
  /// Gets the index.
  inline unsigned int getIndex() const;
  /// Gets the rounding edges.
  inline const MFEdge &getEdges() const;
  /// Gets the rounding faces.
  inline const MFFace &getFaces() const;
  /// Gets the data.
  inline VData &data();
  /// Gets the data.
  inline const VData &data() const;
  /// Gets the data F from VData which must be of type clist<tlist<T...> >.
  template<class F>  
  F &getData();
  /// Gets optionally the data F from VData which must be of type clist<tlist<T...> >.
  template<class F>  
  F &ogetData();
  /// Returns true if is an isolated vertex.
  inline bool isIsolated() const;
  
private:
  const unsigned int _index;
  MFEdge _edges;
  MFFace _faces;
  VData _data;

  /// Constructor.
  SFVertex(unsigned int i);
  /// Destructor.
  ~SFVertex();
  
  /// Adds an edge.
  void addEdge(SFEdge<EData, FData, VData, readOnly> *edge);
  /// Adds a face.
  void addFace(SFFace<FData, VData, EData, readOnly> *face);
  
  /// Removes an edge.
  void removeEdge(SFEdge<EData, FData, VData, readOnly> *edge);
  /// Adds a face.
  void removeFace(SFFace<FData, VData, EData, readOnly> *face);
};

/// operator<< overloaded for SFEdge class.
template<class EData, class FData, class VData, bool readOnly>
std::ostream& operator<<(std::ostream& o, const SFVertex<VData, EData, FData, readOnly> &v);

}
}

#include "MESH_SFVertex.inl"

#endif
