////////////////////////////////////////////////////////////////////////////////
//                            MESH_SFVertex.inl                               //
////////////////////////////////////////////////////////////////////////////////

namespace X3DTK {
namespace MESH {

template<class VData, class EData, class FData, bool readOnly>
unsigned int SFVertex<VData, EData, FData, readOnly>::getIndex() const 
{
  return _index;
}

template<class VData, class EData, class FData, bool readOnly>
const typename SFVertex<VData, EData, FData, readOnly>::MFEdge &SFVertex<VData, EData, FData, readOnly>::getEdges() const 
{
  return _edges;
}

template<class VData, class EData, class FData, bool readOnly>
const typename SFVertex<VData, EData, FData, readOnly>::MFFace &SFVertex<VData, EData, FData, readOnly>::getFaces() const 
{
  return _faces;
}

template<class VData, class EData, class FData, bool readOnly>
VData &SFVertex<VData, EData, FData, readOnly>::data() 
{
  return _data;
}

template<class VData, class EData, class FData, bool readOnly>
const VData &SFVertex<VData, EData, FData, readOnly>::data() const 
{
  return _data;
}

template<class VData, class EData, class FData, bool readOnly>
template<class F>  
F &SFVertex<VData, EData, FData, readOnly>::getData()
{
  return _data.template get<F, false>();
}

template<class VData, class EData, class FData, bool readOnly>
template<class F>  
F &SFVertex<VData, EData, FData, readOnly>::ogetData()
{
  return _data.template get<F, true>();
}

template<class VData, class EData, class FData, bool readOnly>
SFVertex<VData, EData, FData, readOnly>::SFVertex<VData, EData, FData, readOnly>(unsigned int i)
: _index(i) 
{
}

template<class VData, class EData, class FData, bool readOnly>
SFVertex<VData, EData, FData, readOnly>::~SFVertex<VData, EData, FData, readOnly>()
{
}

template<class VData, class EData, class FData, bool readOnly>
bool SFVertex<VData, EData, FData, readOnly>::isIsolated() const
{
  return (_edges.size() == 0);
}

template<class VData, class EData, class FData, bool readOnly>
void SFVertex<VData, EData, FData, readOnly>::addEdge(SFEdge<EData, FData, VData, readOnly> *edge) 
{
  _edges.push_back(edge);
}

template<class VData, class EData, class FData, bool readOnly>
void SFVertex<VData, EData, FData, readOnly>::addFace(SFFace<FData, VData, EData, readOnly> *face) 
{
  _faces.push_back(face);
}

template<class VData, class EData, class FData, bool readOnly>
void SFVertex<VData, EData, FData, readOnly>::removeEdge(SFEdge<EData, FData, VData, readOnly> *edge)
{
  typename MFEdge::iterator it = std::find(_edges.begin(), _edges.end(), edge);
  if (it != _edges.end())
    _edges.erase(it);
}

template<class VData, class EData, class FData, bool readOnly>
void SFVertex<VData, EData, FData, readOnly>::removeFace(SFFace<FData, VData, EData, readOnly> *face)
{
  typename MFFace::iterator it = std::find(_faces.begin(), _faces.end(), face);
  if (it != _faces.end())
    _faces.erase(it);
}

template<class EData, class FData, class VData, bool readOnly>
std::ostream& operator<<(std::ostream& o, const SFVertex<VData, EData, FData, readOnly> &v)
{
  o << "[ " << v.getIndex() << " ]";
  return o;
}
  
}
}  
