////////////////////////////////////////////////////////////////////////////////
//                            MESH_SceneGraphTypes.inl                        //
////////////////////////////////////////////////////////////////////////////////

namespace X3DTK {
namespace MESH {

template<class VData, class EData, class FData, bool readOnly>
EntityMFVertex<VData, EData, FData, readOnly>::EntityMFVertex() 
: std::vector<SFVertex<VData, EData, FData, readOnly> *>()
{
}

template<class VData, class EData, class FData>
EntityMFVertex<VData, EData, FData, false>::EntityMFVertex() 
: std::list<SFVertex<VData, EData, FData, false> *>()
{
}



template<class VData, class EData, class FData, bool readOnly>
MeshMFVertex<VData, EData, FData, readOnly>::MeshMFVertex() 
: std::vector<SFVertex<VData, EData, FData, readOnly> *>()
{
}

template<class VData, class EData, class FData>
MeshMFVertex<VData, EData, FData, false>::MeshMFVertex() 
: std::map<unsigned int, SFVertex<VData, EData, FData, false> *>()
{
}

template<class EData, class FData, class VData, bool readOnly>
EntityMFEdge<EData, FData, VData, readOnly>::EntityMFEdge() 
: std::vector<SFEdge<EData, FData, VData, readOnly> *>() 
{
}

template<class EData, class FData, class VData>
EntityMFEdge<EData, FData, VData, false>::EntityMFEdge() 
: std::list<SFEdge<EData, FData, VData, false> *>()
{
}

template<class EData, class FData, class VData, bool readOnly>
EntityMFEdge<EData, FData, VData, readOnly>::EntityMFEdge(const std::vector<SFEdge<EData, FData, VData, readOnly> *> &vector, bool ccw)
: std::vector<SFEdge<EData, FData, VData, readOnly> *>(vector)
{
  if (!ccw)
  {
    unsigned int last = vector.size() - 1;
    unsigned int mid = (last + 1)/2;
    for (unsigned int i = 0; i < mid; ++i)
    {
      SFEdge<EData, FData, VData, readOnly> *temp = (*this)[i];
      (*this)[i] = (*this)[last - i];
      (*this)[last - i] = temp;
    }
  }  
}



template<class EData, class FData, class VData, bool readOnly>
MeshMFEdge<EData, FData, VData, readOnly>::MeshMFEdge() 
: std::vector<SFEdge<EData, FData, VData, readOnly> *>() 
{
}

template<class EData, class FData, class VData>
MeshMFEdge<EData, FData, VData, false>::MeshMFEdge() 
: std::set<SFEdge<EData, FData, VData, false> *>()
{
}

template<class FData, class VData, class EData, bool readOnly>
EntityMFFace<FData, VData, EData, readOnly>::EntityMFFace() 
: std::vector<SFFace<FData, VData, EData, readOnly> *>() 
{
}

template<class FData, class VData, class EData>
EntityMFFace<FData, VData, EData, false>::EntityMFFace() 
: std::list<SFFace<FData, VData, EData, false> *>() 
{
}



template<class FData, class VData, class EData, bool readOnly>
MeshMFFace<FData, VData, EData, readOnly>::MeshMFFace() 
: std::vector<SFFace<FData, VData, EData, readOnly> *>() 
{
}

template<class FData, class VData, class EData>
MeshMFFace<FData, VData, EData, false>::MeshMFFace() 
: std::map<unsigned int, SFFace<FData, VData, EData, false> *>() 
{
}

}
}

