namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
StatsCalculator<MData, VData, EData, FData, readOnly>::StatsCalculator()
: X3DOnePassProcessor()
{
  stateVariables = GraphTraversal::getInstanceOf<StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly> >();

  graphTraversal = new DFSGraphTraversal();
  graphTraversal->setComponentVisitor(new StatsCalculatorCoreVisitor<MData, VData, EData, FData, readOnly>());
}

template<class MData, class VData, class EData, class FData, bool readOnly>
StatsCalculator<MData, VData, EData, FData, readOnly>::~StatsCalculator()
{
  GraphTraversal::removeInstanceOf<StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly> >();
  delete graphTraversal;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void StatsCalculator<MData, VData, EData, FData, readOnly>::calculate(X3DNode *N)
{
  stateVariables->init();
  graphTraversal->traverse(N);
  stateVariables->finish(); 
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void StatsCalculator<MData, VData, EData, FData, readOnly>::setSelectedNodes(const MFNode &selectedNodes)
{
  stateVariables->setSelectedNodes(selectedNodes);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
MFNode StatsCalculator<MData, VData, EData, FData, readOnly>::getSelectedNodes() const
{
  return stateVariables->getSelectedNodes();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
unsigned int StatsCalculator<MData, VData, EData, FData, readOnly>::getModelFaces() const
{
  return stateVariables->getModelFaces();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
unsigned int StatsCalculator<MData, VData, EData, FData, readOnly>::getModelVertices() const
{
  return stateVariables->getModelVertices();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
unsigned int StatsCalculator<MData, VData, EData, FData, readOnly>::getModelEdges() const
{
  return stateVariables->getModelEdges();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
unsigned int StatsCalculator<MData, VData, EData, FData, readOnly>::getFacesOf(X3DNode *N) const
{
  return stateVariables->getFacesOf(N);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
unsigned int StatsCalculator<MData, VData, EData, FData, readOnly>::getVerticesOf(X3DNode *N) const
{
  return stateVariables->getVerticesOf(N);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
unsigned int StatsCalculator<MData, VData, EData, FData, readOnly>::getEdgesOf(X3DNode *N) const
{
  return stateVariables->getEdgesOf(N);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const SFMatrix34f &StatsCalculator<MData, VData, EData, FData, readOnly>::getMatrixOf(X3DNode *N)
{
  return stateVariables->getMatrixOf(N);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const MeshInfo<MData, VData, EData, FData, readOnly> &StatsCalculator<MData, VData, EData, FData, readOnly>::getModelMeshInfo() const
{
  return stateVariables->getModelMeshInfo();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const std::list<std::pair<X3DNode *, MeshInfo<MData, VData, EData, FData, readOnly> > > &StatsCalculator<MData, VData, EData, FData, readOnly>::getMeshInfoList() const
{
  return stateVariables->getMeshInfoList();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const VertexInfo<VData, EData, FData, readOnly> &StatsCalculator<MData, VData, EData, FData, readOnly>::getModelVertexInfo() const
{
  return stateVariables->getModelVertexInfo();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const std::list<std::pair<X3DNode *, VertexInfo<VData, EData, FData, readOnly> > > &StatsCalculator<MData, VData, EData, FData, readOnly>::getVertexInfoList() const
{
  return stateVariables->getVertexInfoList();
}


}
}  
