namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
StatsCalculatorCoreVisitor<MData, VData, EData, FData, readOnly>::StatsCalculatorCoreVisitor()
{
  // Enter and leave functions.
  defineEnterFunction(&StatsCalculatorCoreVisitor::enterMesh);
  defineEnterFunction(&StatsCalculatorCoreVisitor::enterVertex);
  defineWalkOnFunction(&StatsCalculatorCoreVisitor::walkOnShape);
  
  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly> >();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
bool StatsCalculatorCoreVisitor<MData, VData, EData, FData, readOnly>::walkOnShape(Shape *, SFAbstractNode child) const
{
  MFNode selectedNodes = stateVariables->getSelectedNodes();
  return (selectedNodes.empty() || (std::find(selectedNodes.begin(), selectedNodes.end(), child) != selectedNodes.end()));
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void StatsCalculatorCoreVisitor<MData, VData, EData, FData, readOnly>::enterMesh(Mesh<MData, VData, EData, FData, readOnly> *M) const
{
  if (!stateVariables->getNode(M))
  { 
    stateVariables->setCurrentMesh(M);
  
    stateVariables->addFaces(M->getFaces().size());
    stateVariables->addEdges(M->getEdges().size());

    MFAbstractNode parents = M->getParentList();
    unsigned int size = 0;
    for (MFAbstractNode::const_iterator itParent = parents.begin(); itParent != parents.end(); ++itParent)
    {
      if (dynamic_cast<MESH::Shape *>(*itParent) != 0)
        ++size;
    }
    stateVariables->setInstances(size);
  
    stateVariables->addNode(M);
  }
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void StatsCalculatorCoreVisitor<MData, VData, EData, FData, readOnly>::enterVertex(Vertex<VData, EData, FData, readOnly> *V) const
{
  if (!stateVariables->getNode(V))
  {
    stateVariables->addVertices(V->getVertices().size());
    
    stateVariables->addNode(V);
  }
}

}
}  
