namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::StatsCalculatorStateVariables()
: StateVariables()
{
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::init()
{
  _modelMeshInfo.init();
  _modelVertexInfo.init();
  _meshInfoList.clear();
  _vertexInfoList.clear();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::finish()
{
  _nodeList.clear();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::addNode(SFAbstractNode N)
{
  _nodeList.push_back(N);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
bool StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::getNode(SFAbstractNode N) const
{
  std::list<SFAbstractNode>::const_iterator it = std::find(_nodeList.begin(), _nodeList.end(), N);
  return (it != _nodeList.end());
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::setCurrentMesh(Mesh<MData, VData, EData, FData, readOnly> *mesh)
{
  _currentMesh = mesh;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::addFaces(unsigned int faces)
{
  _modelMeshInfo.addFaces(faces);

  for (typename std::list<std::pair<X3DNode *, MeshInfo<MData, VData, EData, FData, readOnly> > >::iterator it = _meshInfoList.begin(); it != _meshInfoList.end(); ++it)
  {
    if ((*it).first == _currentMesh)
    {
      (*it).second.addFaces(faces);
      return;
    }  
  }
 
  _meshInfoList.push_back(std::pair<X3DNode *, MeshInfo<MData, VData, EData, FData, readOnly> >(_currentMesh, MeshInfo<MData, VData, EData, FData, readOnly>()));
  _meshInfoList.back().second.addFaces(faces);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::addVertices(unsigned int vertices)
{
  _modelVertexInfo.addVertices(vertices);

  for (typename std::list<std::pair<X3DNode *, VertexInfo<VData, EData, FData, readOnly> > >::iterator it = _vertexInfoList.begin(); it != _vertexInfoList.end(); ++it)
  {
    if ((*it).first == _currentMesh)
    {
      (*it).second.addVertices(vertices);
      return;
    }  
  }
 
  _vertexInfoList.push_back(std::pair<X3DNode *, VertexInfo<VData, EData, FData, readOnly> >(_currentMesh, VertexInfo<VData, EData, FData, readOnly>()));
  _vertexInfoList.back().second.addVertices(vertices);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::addEdges(unsigned int edges)
{
  _modelMeshInfo.addEdges(edges);

  for (typename std::list<std::pair<X3DNode *, MeshInfo<MData, VData, EData, FData, readOnly> > >::iterator it = _meshInfoList.begin(); it != _meshInfoList.end(); ++it)
  {
    if ((*it).first == _currentMesh)
    {
      (*it).second.addEdges(edges);
      return;
    }  
  }
 
  _meshInfoList.push_back(std::pair<X3DNode *, MeshInfo<MData, VData, EData, FData, readOnly> >(_currentMesh, MeshInfo<MData, VData, EData, FData, readOnly>()));
  _meshInfoList.back().second.addEdges(edges);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::setInstances(unsigned int instances)
{
  for (typename std::list<std::pair<X3DNode *, MeshInfo<MData, VData, EData, FData, readOnly> > >::iterator it = _meshInfoList.begin(); it != _meshInfoList.end(); ++it)
  {
    if ((*it).first == _currentMesh)
    {
      (*it).second.setInstances(instances);
      return;
    }  
  }
 
  _meshInfoList.push_back(std::pair<X3DNode *, MeshInfo<MData, VData, EData, FData, readOnly> >(_currentMesh, MeshInfo<MData, VData, EData, FData, readOnly>()));
  _meshInfoList.back().second.setInstances(instances);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
unsigned int StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::getModelFaces() const
{
  return _modelMeshInfo.getFaces();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
unsigned int StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::getModelVertices() const
{
  return _modelVertexInfo.getVertices();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
unsigned int StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::getModelEdges() const
{
  return _modelMeshInfo.getEdges();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
unsigned int StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::getFacesOf(X3DNode *N)
{
  for (typename std::list<std::pair<X3DNode *, MeshInfo<MData, VData, EData, FData, readOnly> > >::iterator it = _meshInfoList.begin(); it != _meshInfoList.end(); ++it)
  {
    if ((*it).first == N)
      return (*it).second.getFaces();
  }
 
  _meshInfoList.push_back(std::pair<X3DNode *, MeshInfo<MData, VData, EData, FData, readOnly> >(N, MeshInfo<MData, VData, EData, FData, readOnly>()));
  return _meshInfoList.back().second.getFaces();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
unsigned int StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::getVerticesOf(X3DNode *N)
{
  for (typename std::list<std::pair<X3DNode *, VertexInfo<VData, EData, FData, readOnly> > >::iterator it = _vertexInfoList.begin(); it != _vertexInfoList.end(); ++it)
  {
    if ((*it).first == N)
      return (*it).second.getVertices();
  }
 
  _vertexInfoList.push_back(std::pair<X3DNode *, VertexInfo<VData, EData, FData, readOnly> >(N, VertexInfo<VData, EData, FData, readOnly>()));
  return _vertexInfoList.back().second.getVertices();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
unsigned int StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::getEdgesOf(X3DNode *N)
{
  for (typename std::list<std::pair<X3DNode *, MeshInfo<MData, VData, EData, FData, readOnly> > >::iterator it = _meshInfoList.begin(); it != _meshInfoList.end(); ++it)
  {
    if ((*it).first == N)
      return (*it).second.getEdges();
  }
 
  _meshInfoList.push_back(std::pair<X3DNode *, MeshInfo<MData, VData, EData, FData, readOnly> >(N, MeshInfo<MData, VData, EData, FData, readOnly>()));
  return _meshInfoList.back().second.getEdges();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const MeshInfo<MData, VData, EData, FData, readOnly> &StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::getModelMeshInfo() const
{
  return _modelMeshInfo;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const std::list<std::pair<X3DNode *, MeshInfo<MData, VData, EData, FData, readOnly> > > &StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::getMeshInfoList() const
{
  return _meshInfoList;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const VertexInfo<VData, EData, FData, readOnly> &StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::getModelVertexInfo() const
{
  return _modelVertexInfo;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const std::list<std::pair<X3DNode *, VertexInfo<VData, EData, FData, readOnly> > > &StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::getVertexInfoList() const
{
  return _vertexInfoList;
}


template<class MData, class VData, class EData, class FData, bool readOnly>
void StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::setSelectedNodes(const MFNode &selectedNodes)
{
  _selectedNodes = selectedNodes;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
MFNode StatsCalculatorStateVariables<MData, VData, EData, FData, readOnly>::getSelectedNodes() const
{
  return _selectedNodes;
}

}
}
