/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_VERTEXSET_H
#define X3DTK_MESH_VERTEXSET_H

#include "MESH_X3DNode.h"
#include "MESH_SFVertex.h"
#include "MESH_MeshData.h"

namespace X3DTK {
namespace MESH {

/*! \brief Template class declared in <b><X3DTK/MESH/scenegraph.h></b> defining a vertex set,
 *  close to the implementation of X3D::Coordinate for the MESH scene graph.
 *
 *  \ingroup MeshSceneGraph
 */

template<class VData, class EData, class FData, bool readOnly = true>
class Vertex : public X3DNode
{
public:
  typedef MeshMFVertex<VData, EData, FData, readOnly> MFVertex;

  /// Constructor.
  Vertex();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~Vertex();
    
  /// Creates a new vertex.
  SFVertex<VData, EData, FData, readOnly> *createVertex();
  /// Creates a new vertex at the position i.
  SFVertex<VData, EData, FData, readOnly> *createVertex(unsigned int i);

  /// Removes vertex.
  void removeVertex(SFVertex<VData, EData, FData, readOnly> *vertex);
  /// Removes vertex at index i.
  void removeVertex(unsigned int i);
  
  /// Gets the vertices.
  inline const MFVertex &getVertices() const;

protected:
  /// Copy constructor.
  Vertex(const Vertex &S);

private:
  MFVertex _vertices; 
};  

/// Vertex for the default X3DNode nodes containing all the geometry and topology 
/// informations of an X3D scene.
typedef Vertex<VertexData, EdgeData, FaceData, true> DefaultVertex;

}
}

#include "MESH_Vertex.inl"

#endif
