namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
VerticesDrawer<MData, VData, EData, FData, readOnly>::VerticesDrawer()
: X3DOnePassProcessor()
{
  stateVariables = GraphTraversal::getInstanceOf<VerticesDrawerStateVariables<MData, VData, EData, FData, readOnly> >();

  graphTraversal = new DFSGraphTraversal();
  graphTraversal->setComponentVisitor(new VerticesDrawerCoreVisitor<MData, VData, EData, FData, readOnly>());
}

template<class MData, class VData, class EData, class FData, bool readOnly>
VerticesDrawer<MData, VData, EData, FData, readOnly>::~VerticesDrawer()
{
  GraphTraversal::removeInstanceOf<VerticesDrawerStateVariables<MData, VData, EData, FData, readOnly> >();
  delete graphTraversal;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void VerticesDrawer<MData, VData, EData, FData, readOnly>::draw(X3DNode *N, bool selection)
{
  stateVariables->setSelectionMode(selection);
  stateVariables->init();
  graphTraversal->traverse(N);
  stateVariables->finish();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
Mesh<MData, VData, EData, FData, readOnly> *VerticesDrawer<MData, VData, EData, FData, readOnly>::getMesh(int i) const 
{
  return stateVariables->getMesh(i);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
const SFMatrix34f &VerticesDrawer<MData, VData, EData, FData, readOnly>::getMatrix(int i) const 
{
  return stateVariables->getMatrix(i);
}

}
}
