/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef MESH_VERTICESDRAWERCOREVISITOR_H
#define MESH_VERTICESDRAWERCOREVISITOR_H

#include "MESH_VerticesDrawerStateVariables.h"
#include "MESH_CoreVisitor.h"
#include "MESH_Mesh.h"
#include "MESH_Transform.h"

#include <GL/gl.h>

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/verticesdrawer.h></b> providing a visitor of the 
 *  Core component of the MESH::VerticesDrawer processor.
 *
 *  \ingroup MESH
 */

template<class MData, class VData, class EData, class FData, bool readOnly>
class VerticesDrawerCoreVisitor : public CoreVisitor
{
public:
  /// Constructor.
  VerticesDrawerCoreVisitor();

  /// Enters a Mesh.
  virtual void enterMesh(Mesh<MData, VData, EData, FData, readOnly> *M) const;
  /// Enters a Transform : push matrix
  virtual void enterTransform(Transform *T) const;
  /// Leaves a Transform : pop matrix
  virtual void leaveTransform(Transform *) const;

protected:
  VerticesDrawerStateVariables<MData, VData, EData, FData, readOnly> *stateVariables;
};

}
}

#include "MESH_VerticesDrawerCoreVisitor.inl"

#endif
