namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
VerticesDrawerCoreVisitor<MData, VData, EData, FData, readOnly>::VerticesDrawerCoreVisitor()
{
  // Enter and leave functions.
  defineEnterFunction(&VerticesDrawerCoreVisitor::enterTransform);
  defineEnterFunction(&VerticesDrawerCoreVisitor::enterMesh);
  defineLeaveFunction(&VerticesDrawerCoreVisitor::leaveTransform);

  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<VerticesDrawerStateVariables<MData, VData, EData, FData, readOnly> >();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void VerticesDrawerCoreVisitor<MData, VData, EData, FData, readOnly>::enterMesh(Mesh<MData, VData, EData, FData, readOnly> *M) const
{
  stateVariables->storeMesh(M);
  int meshId = stateVariables->getCurrentMeshId();

  if (stateVariables->getSelectionMode())
    glPushName(meshId);

  glColor3f(1.0f, 0.9f, 0.8f);
  if (stateVariables->getSelectionMode())
    glPointSize(5.0f);
  else
    glPointSize(1.0f);
   
  glDisable(GL_LIGHTING);  

  const typename Mesh<MData, VData, EData, FData, readOnly>::MFVertex &vertices = M->getVertices();

  if (stateVariables->getSelectionMode())
  {
    for (typename Mesh<MData, VData, EData, FData, readOnly>::MFVertex::const_iterator v = vertices.begin(), end = vertices.end() ; v != end; ++v)
    {
      glPushName((*v)->getIndex());
      SFPoint3f point = (*v)->data().getPoint();
      glRasterPos3fv(point.f_data());  
      glPopName();
    }
    glPopName();
  }
  else
  {
    glBegin(GL_POINTS);
    for (typename Mesh<MData, VData, EData, FData, readOnly>::MFVertex::const_iterator v = vertices.begin(), end = vertices.end() ; v != end; ++v)
    {
      SFPoint3f point = (*v)->data().getPoint();
      glVertex3fv(point.f_data());
    }
    glEnd();
  } 
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void VerticesDrawerCoreVisitor<MData, VData, EData, FData, readOnly>::enterTransform(Transform *T) const
{
  glMatrixMode(GL_MODELVIEW);
  stateVariables->pushMatrix(T->getTransform());
  glPushMatrix();
  glMultMatrixf(T->getTransform().toFloat16());
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void VerticesDrawerCoreVisitor<MData, VData, EData, FData, readOnly>::leaveTransform(Transform *) const
{
  glMatrixMode(GL_MODELVIEW);
  stateVariables->popMatrix();
  glPopMatrix();
}

}
}
