/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_X3DBUILDER_H
#define X3DTK_MESH_X3DBUILDER_H

#include "MESH_SceneGraphTypes.h"
#include "X3DOnePassProcessor.h"
#include "MESH_X3DBuilderStateVariables.h"
#include "MESH_X3DBuilderCoreVisitor.h"
#include "MESH_DFSGraphTraversal.h"
#include "MESH_X3DNode.h"
#include "MESH_Scene.h"
#include "MESH_MeshData.h"
#include "X3D_Scene.h"

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/x3dbuilder.h></b> providing a processor that 
 *  builds an X3D scene graph from the MESH scene graph.
 *
 *  \ingroup MESH
 */
 
template<class MData, class VData, class EData, class FData, bool readOnly>
class X3DBuilder : public X3DOnePassProcessor
{
public:
  /// Constructor.
  X3DBuilder();
  /// Destructor.
  virtual ~X3DBuilder();

  /// Builds the X3D graph scene from the MESH.
  virtual X3D::SFNode build(SFNode N);
  /// Builds the X3D scene from the MESH scene.
  X3D::Scene *build(Scene *S);
  
protected:  
  X3DBuilderStateVariables<MData, VData, EData, FData, readOnly> *stateVariables;
};

/// Default X3DBuilder.
typedef X3DBuilder<MeshData, VertexData, EdgeData, FaceData, true> DefX3DBuilder;

}
}

#include "MESH_X3DBuilder.inl"

#endif

