namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
X3DBuilder<MData, VData, EData, FData, readOnly>::X3DBuilder()
: X3DOnePassProcessor()
{
  graphTraversal = new DFSGraphTraversal();
  graphTraversal->setComponentVisitor(new X3DBuilderCoreVisitor<MData, VData, EData, FData, readOnly>());

  stateVariables = GraphTraversal::getInstanceOf<X3DBuilderStateVariables<MData, VData, EData, FData, readOnly> >();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
X3DBuilder<MData, VData, EData, FData, readOnly>::~X3DBuilder()
{
  GraphTraversal::removeInstanceOf<X3DBuilderStateVariables<MData, VData, EData, FData, readOnly> >();
  delete graphTraversal;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
X3D::SFNode X3DBuilder<MData, VData, EData, FData, readOnly>::build(SFNode N)
{
  stateVariables->init();
  graphTraversal->traverse(N);
  stateVariables->finish();

  return stateVariables->getRoot();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
X3D::Scene *X3DBuilder<MData, VData, EData, FData, readOnly>::build(Scene *S)
{
  return static_cast<X3D::Scene *>(build(static_cast<SFNode>(S)));
}

}
}


