/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_X3DBUILDERVISITOR_H
#define X3DTK_MESH_X3DBUILDERVISITOR_H

#include "MESH_CoreVisitor.h"
#include "MESH_Mesh.h"
#include "MESH_Vertex.h"
#include "MESH_X3DBuilderStateVariables.h"
#include "MESH_Scene.h"
#include "MESH_Shape.h"
#include "MESH_Transform.h"
#include "MESH_Switch.h"
#include "X3D_Scene.h"
#include "X3D_Transform.h"
#include "X3D_Switch.h"
#include "X3D_Shape.h"
#include "X3D_IndexedFaceSet.h"
#include "X3D_Coordinate.h"
#include "X3D_X3DAppearanceNode.h"
#include "X3D_Normal.h"
#include "X3D_ColorRGBA.h"
#include "X3D_TextureCoordinate.h"

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/x3dbuilder.h></b> providing a visitor of the 
 *  Core component of the MESH::X3DBuilder processor.
 *
 *  \ingroup MESH
 */
 
template<class MData, class VData, class EData, class FData, bool readOnly>
class X3DBuilderCoreVisitor : public CoreVisitor
{
public:
  /// Constructor.
  X3DBuilderCoreVisitor();

  /// Enters a Scene.
  virtual void enterScene(Scene *S) const;
  /// Enters a Transform.
  virtual void enterTransform(Transform *T) const;
  /// Enters a Switch.
  virtual void enterSwitch(Switch *S) const;
  /// Enters a Shape.
  virtual void enterShape(Shape *S) const;
  /// Enters a Mesh.
  virtual void enterMesh(Mesh<MData, VData, EData, FData, readOnly> *M) const;
  /// Enters a Vertex.
  virtual void enterVertex(Vertex<VData, EData, FData, readOnly> *V) const;
  /// Leaves an X3DNode.
  virtual void leaveX3DNode(X3DNode *) const;
  
protected:
  X3DBuilderStateVariables<MData, VData, EData, FData, readOnly> *stateVariables; 
};

}
}

#include "MESH_X3DBuilderCoreVisitor.inl"

#endif
