namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
X3DBuilderStateVariables<MData, VData, EData, FData, readOnly>::X3DBuilderStateVariables()
: StateVariables(), _root(0), _cloner(X3DProcessor::getInstanceOf<Cloner>())
{
}

template<class MData, class VData, class EData, class FData, bool readOnly>
X3DBuilderStateVariables<MData, VData, EData, FData, readOnly>::~X3DBuilderStateVariables()
{
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void X3DBuilderStateVariables<MData, VData, EData, FData, readOnly>::init()
{
  _root = 0;
  _nodeStack.push_front(0);
  _cloner->init();
}
template<class MData, class VData, class EData, class FData, bool readOnly>
void X3DBuilderStateVariables<MData, VData, EData, FData, readOnly>::finish()
{
  _nodeStack.clear();
  _nodeCoupleMap.clear();
  _cloner->finish();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void X3DBuilderStateVariables<MData, VData, EData, FData, readOnly>::addCoupleNode(SFNode N, X3D::SFNode NN)
{
  _nodeCoupleMap[N] = NN;  

  // init root node.
  if (_root == 0)
    _root = NN;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void X3DBuilderStateVariables<MData, VData, EData, FData, readOnly>::pushNode(X3D::SFNode N)
{
  X3D::SFNode T = getTop();
  if (T != 0)
    T->addChild(N);
  
  // filling the stack.    
  _nodeStack.push_front(N);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void X3DBuilderStateVariables<MData, VData, EData, FData, readOnly>::popNode()
{
  _nodeStack.pop_front();
}
  
template<class MData, class VData, class EData, class FData, bool readOnly>
X3D::SFNode X3DBuilderStateVariables<MData, VData, EData, FData, readOnly>::getRoot() const 
{
  return _root;
}

template<class MData, class VData, class EData, class FData, bool readOnly>
X3D::SFNode X3DBuilderStateVariables<MData, VData, EData, FData, readOnly>::getTop() const 
{
  return (!_nodeStack.empty() ? _nodeStack.front() : 0);
}

template<class MData, class VData, class EData, class FData, bool readOnly>
X3D::SFNode X3DBuilderStateVariables<MData, VData, EData, FData, readOnly>::getNode(SFNode N) const
{
  std::map<SFNode, X3D::SFNode>::const_iterator it = _nodeCoupleMap.find(N);
  if (it != _nodeCoupleMap.end())
    return (*it).second;
  
  return 0;  
}

template<class MData, class VData, class EData, class FData, bool readOnly>
X3D::X3DNode *X3DBuilderStateVariables<MData, VData, EData, FData, readOnly>::clone(X3D::X3DNode *N)
{
  return static_cast<X3D::X3DNode *>(_cloner->clone(N, false));
}
  
}
}

