/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_WALKER_H
#define X3DTK_WALKER_H

#include "X3DActor.h"
#include "X3DAbstractNode.h"
#include "WalkingFunction.h"

#include <list>

namespace X3DTK {

class SFType;
class X3DComponentWalker;
class Visitor;

/*! \brief Class declared in <b><X3DTK/kernel.h></b> defining 
 *  an actor specialized in the walking of nodes.
 *
 *  \ingroup processor
 */

class Walker : public X3DActor
{
public:  
  /// Constructor.
  Walker();
  /// Destructor.
  virtual ~Walker();
  
  /// Sets the NodeVisitingProxy.
  void setVisitor(Visitor *visitor);  
  /// Sets the X3DComponentCreator.
  void setComponentWalker(X3DComponentWalker *component);
  /// Creates the SFNode from its name.
  inline void walk(SFAbstractNode N) const {_walkingArray[N->getType()->getId()]->walk(N);};
  /// Empties the list of component.
  void reset();
  
  friend Walker *joinWalkers(Walker *N0, Walker *N1);

private:  
  /// Adds a type.
  virtual void addType(const SFType *type);
  /// finds the lowest function in the derivation tree.
  WalkingFunction *getWalkingFunctionOf(const SFType *type) const;
  
  //Dictionary of all the creation functions used.
  WalkingArray _walkingArray;
  std::list<X3DComponentWalker *> _componentList;
  Visitor *_visitor;
};

}

#endif
