/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3DABSTRACTNODE_H
#define X3DTK_X3DABSTRACTNODE_H

#include "X3DTypes.h"
#include "X3DFileElement.h"
#include "SFType.h"

namespace X3DTK {

class FileElement;

/*! \brief Class declared in <b><X3DTK/kernel.h></b> defining the root of the derivation tree.
 *
 *  \ingroup abstractNode
 */

class X3DAbstractNode
{
public:
  /// Constructor.
  X3DAbstractNode();
  /// Clones the node.
  virtual SFAbstractNode clone() const;
  /// Virtual destructor.
  virtual ~X3DAbstractNode();

  /// Sets the name of the node.
  void setName(const SFString &name);

  /// Gets the name of the node. Identical to DEF.
  inline const SFString &getName() const {return _name;};
  /// Gets the SFType name of the node.
  inline SFString getTypeName() const {return _type->getName();};
  /// Gets the SFType of the node.
  inline SFType *getType() const {return _type;};
  /// Gets the list of children.
  virtual MFAbstractNode getChildrenList() const;
  /// Gets the parent list.
  inline MFAbstractNode getParentList() const {return _parentList;};
    
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  virtual bool addChild(const SFAbstractNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  virtual bool setChild(const SFAbstractNode &N);
  /// Removes the child if exists. Returns TRUE if suceeds.
  virtual bool removeChild(const SFAbstractNode &N);

protected:
  /// Copy constructor.
  X3DAbstractNode(const X3DAbstractNode &N);

  /// Method to call at the first line of the body of the constructor, and passing 
  /// in argument the name of the tag of the X3D file.
  inline void defineTypeName(const SFString &name, const SFString &component = SFString(""), const SFString &sceneGraph = SFString("")) {SFType::defineTypeName(&_type, name, component, sceneGraph);};
  /// Adds one reference.
  inline void addOneReference() {_type->addOneReferenceRecursive();};
  /// Adds the child to parent. 
  static void addParentToChild(SFAbstractNode parent, SFAbstractNode child);
  /// Removes the child from parent.
  static void removeParentFromChild(SFAbstractNode parent, SFAbstractNode child);
  
private:   
  SFString _name;
  SFType *_type;
  MFAbstractNode _parentList;
  
  /// Adds a parent.
  void addParent(const SFAbstractNode &N);
  /// Removes a parent.
  bool removeParent(const SFAbstractNode &N);
};

}

#endif
