///////////////////////////////////////////////////////////////////////////////
//                            X3DComponentVisitor.inl                        //
///////////////////////////////////////////////////////////////////////////////

namespace X3DTK {

template<class C, class T> 
void X3DComponentVisitor::defineEnterFunction(void (C::*ptrF)(T *) const)
{
  SFType::beginFunctionDefinition();
  EnterFunction *EF = new EnterFunction(reinterpret_cast<ptrToEnterFunction>(ptrF), this);
  //finding if an occurence of T
  T *t = new T();
  SFString name = t->getTypeName();
  SFComponent *TComponent = t->getType()->getComponent();
  //checking if the node belongs to the component.
  if (TComponent != component)
  {
    SFString sg1, sg2;
    if (component->getSceneGraphName() != "")
      sg1 = " of the " + component->getSceneGraphName() + " scene graph";
    if (t->getType()->getSceneGraphName() != "")
      sg1 = " of the " + t->getType()->getSceneGraphName() + " scene graph";  

    cx3d << "warning: defineEnterFunction for " << name << " which doesn't belong to the " << component->getName() << " component" << sg1 << "," << std::endl;
    cx3d << "         but to the " << t->getType()->getComponentName() << " component" << sg2 << std::endl;
  }
  else
    _enterDict.insert(std::pair<SFString, EnterFunction *>(t->getType()->getEncodedName(), EF));

  delete t;
  SFType::endFunctionDefinition();
}
  
template<class C, class T> 
void X3DComponentVisitor::defineWalkOnFunction(bool (C::*ptrF)(T *, SFAbstractNode) const)
{
  SFType::beginFunctionDefinition();
  WalkOnFunction *WF = new WalkOnFunction(reinterpret_cast<ptrToWalkOnFunction>(ptrF), this);
  //finding if an occurence of T
  T *t = new T();
  SFString name = t->getTypeName();
  SFComponent *TComponent = t->getType()->getComponent();
  //checking if the node belongs to the component.
  if (TComponent != component)
  {
    SFString sg1, sg2;
    if (component->getSceneGraphName() != "")
      sg1 = " of the " + component->getSceneGraphName() + " scene graph";
    if (t->getType()->getSceneGraphName() != "")
      sg1 = " of the " + t->getType()->getSceneGraphName() + " scene graph";  

    cx3d << "warning: defineWalkOnFunction for " << name << " which doesn't belong to the " << component->getName() << " component" << sg1 << "," << std::endl;
    cx3d << "         but to the " << t->getType()->getComponentName() << " component" << sg2 << std::endl;    
  }
  else
    _walkonDict.insert(std::pair<SFString, WalkOnFunction *>(t->getType()->getEncodedName(), WF));
    
  delete t;
  SFType::endFunctionDefinition();
};

template<class C, class T> 
void X3DComponentVisitor::defineLeaveFunction(void (C::*ptrF)(T *) const)
{
  SFType::beginFunctionDefinition();
  LeaveFunction *LF = new LeaveFunction(reinterpret_cast<ptrToLeaveFunction>(ptrF), this);
  //finding if an occurence of T
  T *t = new T();
  SFString name = t->getTypeName();
  SFComponent *TComponent = t->getType()->getComponent();
  //checking if the node belongs to the component.
  if (TComponent != component)
  {
    SFString sg1, sg2;
    if (component->getSceneGraphName() != "")
      sg1 = " of the " + component->getSceneGraphName() + " scene graph";
    if (t->getType()->getSceneGraphName() != "")
      sg1 = " of the " + t->getType()->getSceneGraphName() + " scene graph";  

    cx3d << "warning: defineLeaveFunction for " << name << " which doesn't belong to the " << component->getName() << " component" << sg1 << "," << std::endl;
    cx3d << "         but to the " << t->getType()->getComponentName() << " component" << sg2 << std::endl;    
  }
  else
    _leaveDict.insert(std::pair<SFString, LeaveFunction *>(t->getType()->getEncodedName(), LF));
    
  delete t;
  SFType::endFunctionDefinition();
};

}  
