/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3DCOMPONENTWALKER_H
#define X3DTK_X3DCOMPONENTWALKER_H

#include "X3DComponent.h"
#include "WalkingFunction.h"
#include "X3DAbstractNode.h"
#include "Walker.h"
#include "Visitor.h"
#include "SFSceneGraph.h"

namespace X3DTK {

/*! \brief Abstract class declared in <b><X3DTK/kernel.h></b> defining the concept of component walker.
 *
 *  \ingroup processor
 */

class X3DComponentWalker : public X3DComponent
{
public:
  /// Constructor.
  X3DComponentWalker();
  /// Destructor.
  virtual ~X3DComponentWalker() = 0;
  
  /// Sets the NodeVisitingProxy
  void setWalker(Walker *walker);
  /// Sets the NodeVisitingProxy
  void setVisitor(Visitor *visitor);
  
  /// Gets the walking function of name.
  WalkingFunction *getWalkingFunctionOf(const SFType *type) const;
  
protected:  
  /// Reference to the NodeWalkingProxy.
  Walker *walker;
  /// Reference to the NodeVisitingProxy.
  Visitor *visitor;
  /// Defines a new creation function.
  template<class C, class T> 
  void defineWalkingFunction(void (C::*ptrF)(T *) const);
    
private:
  WalkingDict _walkingDict;
};

}

#include "X3DComponentWalker.inl"

#endif
