/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3DIMAGETYPE_H
#define X3DTK_X3DIMAGETYPE_H

#include "Platform.h"

#include <GL/gl.h>

namespace X3DTK {

/*! \brief Class declared in <b><X3DTK/kernel.h></b> providing an interface for the 
 *  different types of images. 
 *
 *  The only method to redefine is load. 
 *
 *  \ingroup image
 */
 
class X3DImageType 
{
public:
  /// Constructor.
  X3DImageType();
  /// Destructor.
  virtual ~X3DImageType();
  
  /// Loads the file. This method has to be redefined.
  virtual bool load(const char *fileName) = 0;  
  /// Resizes the image.
  void resize(unsigned short width, unsigned short height);
  /// Resizes the image in order to be used as an OpenGL texture image. textureTransform 
  /// is the matrix to correct the rendering.
  void resizeGL(float (& textureTransform)[16]);
  
  /// Gets the width.
  inline unsigned short getWidth() const {return _width;};
  /// Gets the height.
  inline unsigned short getHeight() const {return _height;};
  /// Gets the pixel format 
  GLenum getGLType() const;
  /// Gets the array of data.
  inline unsigned char* getData() const {return data;};
  /// Releases the memory.
  void release();

protected:
  /// data.
  unsigned char *data;
  /// Allocates an image.
  bool allocate(int pixelFormat, unsigned short width, unsigned short height);

private:
  int _pixelFormat;    
  unsigned short _width;
  unsigned short _height;
};

}

#endif
