/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3DMULTIPASSPROCESSOR_H
#define X3DTK_X3DMULTIPASSPROCESSOR_H

#include "X3DProcessor.h"
#include "GraphTraversal.h"

#include <map>

namespace X3DTK {

/*! \brief Abstract class declared in <b><X3DTK/kernel.h></b> defining the concept of one pass processor.
 *
 *  \ingroup processor
 */

class X3DMultiPassProcessor : public X3DProcessor
{
  typedef std::map<SFString, GraphTraversal *, ltSFString> MFGraphTraversal;
public:
  /// Constructor.
  X3DMultiPassProcessor();
  /// Destructor.
  virtual ~X3DMultiPassProcessor() = 0;
  
  /// Sets a component Walker.
  virtual void setComponentWalker(const SFString &processor, X3DComponentWalker *component);
  /// Sets a component visitor.
  virtual void setComponentVisitor(const SFString &processor, X3DComponentVisitor *component);

  /// Gets the GraphTraversal.
  GraphTraversal *getGraphTraversalOf(const SFString &processor) const;  

protected:
  MFGraphTraversal graphTraversalMap;  
};

}

#endif
