/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_BBOXUPDATERGROUPINGVISITOR_H
#define X3DTK_X3D_BBOXUPDATERGROUPINGVISITOR_H

#include "X3D_GroupingVisitor.h"
#include "X3D_BBoxUpdaterStateVariables.h"

namespace X3DTK {
namespace X3D {

class StaticGroup;
class X3DGroupingNode;
class Group;
class Transform;
class Switch;

/*! \brief Class declared in <b><X3DTK/X3D/bboxupdater.h></b> providing a visitor of
 *  the Grouping component for the X3D::BBoxUpdater processor.
 *
 *  \ingroup X3D
 */

class BBoxUpdaterGroupingVisitor : public GroupingVisitor
{
public:
  /// Constructor.
  BBoxUpdaterGroupingVisitor();
  /// Destructor.
  virtual ~BBoxUpdaterGroupingVisitor();


  /// Walks on a StaticGroup.
  virtual bool walkOnStaticGroup(StaticGroup *S, SFAbstractNode Child) const;
  /// Walks on a X3DGroupingNode.
  virtual bool walkOnX3DGroupingNode(X3DGroupingNode *N, SFAbstractNode Child) const;

  /// Leaves a StaticGroup.
  virtual void leaveStaticGroup(StaticGroup *S) const;
  /// Leaves a X3DGroupingNode.
  virtual void leaveGroup(Group *G) const;
  /// Leaves a Transform.
  virtual void leaveTransform(Transform *T) const;
  /// Leaves a Switch.
  virtual void leaveSwitch(Switch *S) const;

protected:
  BBoxUpdaterStateVariables *stateVariables;
};

}
}

#endif
