/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_DIRECTIONALLIGHT
#define X3DTK_X3D_DIRECTIONALLIGHT

#include "X3DTypes.h"
#include "X3D_X3DLightNode.h"

namespace X3DTK {
namespace X3D {

/*! \brief Base class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/lighting.html">
 *  Lighting component</a> defining a DirectionalLight
 *  (<a href="../../doc/X3DSpecs/Part01/components/lighting.html#DirectionalLight">
 *  documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */

class DirectionalLight : public X3DLightNode
{
public:
  /// Constructor.
  DirectionalLight();
  /// Constructs a DirectionalLight from its attributes.
  DirectionalLight(const SFFloat       &ambientIntensity,
                   const SFColor &color,
                   const SFVec3f &direction,
                   const SFFloat       &intensity,
                   const SFBool        &on);
  /// Clones the node.
  virtual SFAbstractNode clone() const;
  /// Virtual destructor.
  virtual ~DirectionalLight();

  /// Sets the direction attribute.
  void setDirection(const SFVec3f &direction);

  /// Gets the direction attribute.
  inline const SFVec3f &getDirection() const {return _direction;};

  /// Loads the attributes from a X3DFileElement.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString &write(SFString &output) const;

protected:
  /// Copy constructor.
  DirectionalLight(const DirectionalLight &N);

private:
  /// Direction vector of the illumination emanating from the light source.
  SFVec3f _direction;
};

}
}

#endif
