/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_GLBUILDERRENDERINGVISITOR_H
#define X3DTK_X3D_GLBUILDERRENDERINGVISITOR_H

#include "X3D_RenderingVisitor.h"
#include "X3D_GLBuilderStateVariables.h"

namespace X3DTK {
namespace X3D {

class X3DGeometryNode;
class IndexedLineSet;
class PointSet;
class TriangleFanSet;
class TriangleSet;
class TriangleStripSet;

/*! \brief Class declared in <b><X3DTK/X3D/glbuilder.h></b> providing a visitor
 *  of the Rendering component for the X3D::GLBuilder processor.
 *
 *  \ingroup X3D
 */

class GLBuilderRenderingVisitor : public RenderingVisitor
{
public:
  /// Constructor.
  GLBuilderRenderingVisitor();
  /// Destructor.
  virtual ~GLBuilderRenderingVisitor();

  /// enters a IndexedLineSet.
  virtual void enterIndexedLineSet(IndexedLineSet *I) const;
  /// enters a PointSet.
  virtual void enterPointSet(PointSet *P) const;
  /// enters a TriangleFanSet.
  virtual void enterTriangleFanSet(TriangleFanSet *T) const;
  /// enters a TriangleSet.
  virtual void enterTriangleSet(TriangleSet *T) const;
  /// enters a TriangleStripSet.
  virtual void enterTriangleStripSet(TriangleStripSet *T) const;

  /// Walks on an X3DGeometryNode.
  virtual bool walkOnX3DGeometryNode(X3DGeometryNode *N, SFAbstractNode child) const;

protected:
  GLBuilderStateVariables *stateVariables;
};

}
}

#endif
