/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_MESHBUILDER_H
#define X3DTK_X3D_MESHBUILDER_H

#include "X3DOnePassProcessor.h"
#include "X3D_MeshBuilderStateVariables.h"
#include "X3D_DFSGraphTraversal.h"
#include "X3D_MeshBuilderCoreVisitor.h"
#include "X3D_MeshBuilderGeometry3DVisitor.h"
#include "X3D_MeshBuilderGroupingVisitor.h"
#include "X3D_MeshBuilderShapeVisitor.h"
#include "X3D_MeshBuilderRenderingVisitor.h"
#include "X3D_Scene.h"
#include "MESH_Scene.h"
#include "MESH_MeshData.h"

namespace X3DTK {
namespace X3D {

/*! \brief Class declared in <b><X3DTK/X3D/meshbuilder.h></b> providing a processor
 *  that builds a MESH scene graph from the X3D scene graph.
 *
 *  \ingroup X3D
 */
 
template<class MData, class VData, class EData, class FData, bool readOnly>
class MeshBuilder : public X3DOnePassProcessor
{     
public:
  /// Constructor. 
  MeshBuilder();
  /// Virtual destructor.
  virtual ~MeshBuilder();
    
  /// Builds the mesh from N.
  virtual MESH::SFNode build(SFNode N);
  
  /// Builds the mesh scene from S.
  MESH::Scene *build(Scene *S);
  
private:
  MeshBuilderStateVariables *stateVariables;
};

typedef MeshBuilder<MESH::MeshData, MESH::VertexData, MESH::EdgeData, MESH::FaceData, true> DefMeshBuilder;

}
}

#include "X3D_MeshBuilder.inl"

#endif
