////////////////////////////////////////////////////////////////////////////////
//                            X3D_MeshBuilder.inl                             //
////////////////////////////////////////////////////////////////////////////////

namespace X3DTK {
namespace X3D {

template<class MData, class VData, class EData, class FData, bool readOnly>
MeshBuilder<MData, VData, EData, FData, readOnly>::MeshBuilder() 
{
  stateVariables = GraphTraversal::getInstanceOf<MeshBuilderStateVariables>();
  graphTraversal = new DFSGraphTraversal();

  graphTraversal->setComponentVisitor(new MeshBuilderCoreVisitor());
  graphTraversal->setComponentVisitor(new MeshBuilderGeometry3DVisitor<MData, VData, EData, FData, readOnly>());
  graphTraversal->setComponentVisitor(new MeshBuilderGroupingVisitor());
  graphTraversal->setComponentVisitor(new MeshBuilderShapeVisitor());
  graphTraversal->setComponentVisitor(new MeshBuilderRenderingVisitor<MData, VData, EData, FData, readOnly>());
}

template<class MData, class VData, class EData, class FData, bool readOnly>
MeshBuilder<MData, VData, EData, FData, readOnly>::~MeshBuilder() 
{
  GraphTraversal::removeInstanceOf<MeshBuilderStateVariables>();

  delete graphTraversal;
};

template<class MData, class VData, class EData, class FData, bool readOnly> 
MESH::SFNode MeshBuilder<MData, VData, EData, FData, readOnly>::build(SFNode N)
{
  stateVariables->init();
  graphTraversal->traverse(N);
  stateVariables->finish();
  
  return stateVariables->getRoot();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
MESH::Scene *MeshBuilder<MData, VData, EData, FData, readOnly>::build(Scene *S)
{
  return static_cast<MESH::Scene *>(build(static_cast<SFNode>(S)));
};
  
}
}  
