/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_MESHBUILDERGROUPINGVISITOR_H
#define X3DTK_X3D_MESHBUILDERGROUPINGVISITOR_H

#include "X3D_GroupingVisitor.h"
#include "X3D_MeshBuilderStateVariables.h"

namespace X3DTK {
namespace X3D {

class X3DChildNode;
class X3DGroupingNode;
class StaticGroup;
class Transform;
class Switch;

/*! \brief Class declared in <b><X3DTK/X3D/meshbuilder.h></b> providing a visitor
 *  of the Grouping component for the X3D::MeshBuilder processor.
 *
 *  \ingroup X3D
 */

class MeshBuilderGroupingVisitor : public GroupingVisitor
{
public:
  /// Constructor.
  MeshBuilderGroupingVisitor();
  /// Destructor.
  virtual ~MeshBuilderGroupingVisitor();
  
  /// Enters an X3DChildNode.
  virtual void enterX3DChildNode(X3DChildNode *N) const;
  /// Enters a Group.
  virtual void enterX3DGroupingNode(X3DGroupingNode *N) const;
  /// Enters a StaticGroup.
  virtual void enterStaticGroup(StaticGroup *S) const;
  /// Enters a Transform.
  virtual void enterTransform(Transform *T) const;
  /// Enters a Switch.
  virtual void enterSwitch(Switch *S) const;
  /// Leaves an X3DChildNode.
  virtual void leaveX3DChildNode(X3DChildNode *N) const;
  /// Leaves an X3DGroupingNode.
  virtual void leaveX3DGroupingNode(X3DGroupingNode *N) const;


protected:
  MeshBuilderStateVariables *stateVariables;
};

}
}

#endif
