/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_POINTSET_H
#define X3DTK_X3D_POINTSET_H

#include "X3DTypes.h"
#include "X3D_X3DGeometryNode.h"

namespace X3DTK {
namespace X3D {

class X3DColorNode;
class X3DCoordinateNode;

/*! \brief Class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/rendering.html">
 *  Rendering component</a> defining a PointSet
 *  (<a href="../../doc/X3DSpecs/Part01/components/rendering.html#PointSet">
 *  documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */
 
class PointSet : public X3DGeometryNode
{
public:
  /// Constructor.
  PointSet();
  /// Constructs a PointSet from its attributes.
  PointSet(X3DColorNode *color,
           X3DCoordinateNode *coord);
  /// Clones the node.
  virtual SFAbstractNode clone() const;
  /// Destructor.
  virtual ~PointSet();
  
  /// Sets the Color attribute.
  void setColor(X3DColorNode *color);
  /// Sets the Coord attribute.
  void setCoord(X3DCoordinateNode *coord);
  
  /// Gets the Color attribute.
  inline SFNode getColor() const {return _color;};
  /// Gets the Coord attribute.
  inline SFNode getCoord() const {return _coord;};
  
  /// Gets the children list.
  virtual MFAbstractNode getChildrenList() const;
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  virtual bool addChild(const SFAbstractNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  virtual bool setChild(const SFAbstractNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  virtual bool removeChild(const SFAbstractNode &N);

protected:
  /// Copy constructor. 
  PointSet(const PointSet &P);
  
private:  
  /// Color node.
  SFNode _color; 
  /// Coord node. 
  SFNode _coord;  
};

}
}

#endif
