////////////////////////////////////////////////////////////////////////////////
//                            X3D_X3DComponentCreator.inl                     //
////////////////////////////////////////////////////////////////////////////////

namespace X3DTK {
namespace X3D {

template<class T>
void X3DComponentCreator::defineNode()
{
  defineCreationFunction<T, X3DComponentCreator>(&X3DComponentCreator::create<T>);
}

template<class T>
SFNode X3DComponentCreator::create() const
{
  return new T();
}

template<class T, class C> 
void X3DComponentCreator::defineCreationFunction(SFNode (C::*ptrF)() const)
{
  SFType::beginFunctionDefinition();
  SFNode t = new T();
  SFString name = t->getTypeName();
  SFComponent *TComponent = t->getType()->getComponent();
  //checking if the node belongs to the component.
  if (TComponent != component)
  {
    SFString sg1, sg2;
    if (component->getSceneGraphName() != "")
      sg1 = " of the " + component->getSceneGraphName() + " scene graph";
    if (t->getType()->getSceneGraphName() != "")
      sg1 = " of the " + t->getType()->getSceneGraphName() + " scene graph";  

    cx3d << "warning: defineNode for " << name << " which doesn't belong to the " << component->getName() << " component" << sg1 << "," << std::endl;
    cx3d << "         but to the " << t->getType()->getComponentName() << " component" << sg2 << std::endl;    
  }         
  else
  {
    CreationFunction *CF = new CreationFunction(reinterpret_cast<ptrToCreationFunction>(ptrF), this);
    _creationDict.insert(std::pair<SFString, CreationFunction *>(name, CF));
  }
  delete t;
  SFType::endFunctionDefinition();
}

}
}
