#include "GL_NormalRenderer.h"
#include "GL_NormalRendererGeometry3DVisitor.h"
#include "GL_NormalRendererGroupingVisitor.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace GL {

NormalRenderer::NormalRenderer()
{
  // Getting the state variables to initialize its values.
  stateVariables = GraphTraversal::getInstanceOf<NormalRendererStateVariables>();

  // The algorithm for drawing the normals is based upon a DFS graph traversal 
  // of the GL scene graph.
  graphTraversal = new DFSGraphTraversal();
  
  // Setting the component visitor to the tree traversal algorithm.
  graphTraversal->setComponentVisitor(new NormalRendererGeometry3DVisitor());
  graphTraversal->setComponentVisitor(new NormalRendererGroupingVisitor());
}

NormalRenderer::~NormalRenderer()
{
  // Removing the state variables.
  GraphTraversal::removeInstanceOf<NormalRendererStateVariables>();

  delete graphTraversal;
}

void NormalRenderer::setNormalLength(float value)
{
  stateVariables->setNormalLength(value);
}

void NormalRenderer::render(SFNode N) const
{
  glDisable(GL_COLOR_MATERIAL);
  graphTraversal->traverse(N);
}

}
}
