namespace X3DTK {
namespace MESH {

template<class MData, class VData, class EData, class FData, bool readOnly>
MyX3DBuilderCoreVisitor<MData, VData, EData, FData, readOnly>::MyX3DBuilderCoreVisitor()
{
  defineEnterFunction(&MyX3DBuilderCoreVisitor::enterVertex);

  stateVariables = GraphTraversal::getInstanceOf<X3DBuilderStateVariables<MData, VData, EData, FData, readOnly> >();
}

template<class MData, class VData, class EData, class FData, bool readOnly>
void MyX3DBuilderCoreVisitor<MData, VData, EData, FData, readOnly>::enterVertex(Vertex<VData, EData, FData, readOnly> *V) const
{
  // Test of the presence of MESH::VertexWeightData in VData.
  if (!VData::template find<VertexWeightData>())
  {
    // Call to super class function.
    X3DBuilderCoreVisitor<MData, VData, EData, FData, readOnly>::enterVertex(V);
  }
  else
  {
    // Recoding the creation of the node.
    X3D::MyCoordinate *NC = dynamic_cast<X3D::MyCoordinate *>(stateVariables->getNode(V));
    if (NC == 0)
    {
      // Create a new MyCoordinate.
      NC = new X3D::MyCoordinate();
    
      const typename Vertex<VData, EData, FData, readOnly>::MFVertex &vertices = V->getVertices();
      
      // Test the optional presence of MESH::VertexPointData in VData.
      if (VData::template find<MESH::VertexPointData>())
      {
        // Updating Coordinate.
        MFVec3f point;
        // Accessing the MESH::VertexPointData of VData.
        for (typename Vertex<VData, EData, FData, readOnly>::MFVertex::const_iterator v = vertices.begin(); v != vertices.end(); ++v)
          point.push_back(SFVec3f((*v)->template ogetData<MESH::VertexPointData>().getPoint()));
          
        NC->setPoint(point);
      }
      
      MFFloat weight;
      // Accessing the MESH::VertexWeightData of VData.
      for (typename Vertex<VData, EData, FData, readOnly>::MFVertex::const_iterator v = vertices.begin(); v != vertices.end(); ++v)
        weight.push_back((*v)->template ogetData<MESH::VertexWeightData>().getWeight());
          
      NC->setWeight(weight);
     
      // memorizing the couple (V, NC) to avoid to create a new Coordinate 
      // if V is visited a second time.
      stateVariables->addCoupleNode(V, NC);
    }
    stateVariables->pushNode(NC);
  }  
}

}
}
