#include "MESH_MyMeshData.h"
#include "X3D_MyRenderingCreator.h"
#include "X3D_MyMeshBuilderRenderingVisitor.h"
#include "MESH_MyX3DBuilderCoreVisitor.h"

#include <X3DTK/X3D/scenegraph.h>
#include <X3DTK/X3D/meshbuilder.h>
#include <X3DTK/MESH/x3dbuilder.h>
#include <X3DTK/X3D/scenesaver.h>

#include <iostream>

using namespace X3DTK;
using namespace std;

int main(int argc, char *argv[])
{
  if (argc <= 1)
  {
    cerr << "usage: extendedmeshsaver input" << endl;
    exit(0);
  }
  
  // DefaultLoader to load the default X3D Nodes.
  X3D::Loader *loader = X3DLoader::getInstanceOf<X3D::Loader>();
  loader->setComponentCreator(new X3D::MyRenderingCreator());
  
  // Instanciation of the new MeshBuilder.
  X3D::MeshBuilder<MESH::MyMeshData, MESH::MyVertexData, MESH::EdgeData, MESH::MyFaceData, true> *meshbuilder = X3DProcessor::getInstanceOf<X3D::MeshBuilder<MESH::MyMeshData, MESH::MyVertexData, MESH::EdgeData, MESH::MyFaceData, true> >();
  meshbuilder->setComponentVisitor(new X3D::MyMeshBuilderRenderingVisitor<MESH::MyMeshData, MESH::MyVertexData, MESH::EdgeData, MESH::MyFaceData, true>());

  // Instanciation of the new MeshX3DBuilder.
  MESH::X3DBuilder<MESH::MyMeshData, MESH::MyVertexData, MESH::EdgeData, MESH::MyFaceData, true> *meshx3dbuilder = X3DProcessor::getInstanceOf<MESH::X3DBuilder<MESH::MyMeshData, MESH::MyVertexData, MESH::EdgeData, MESH::MyFaceData, true> >();
  meshx3dbuilder->setComponentVisitor(new MESH::MyX3DBuilderCoreVisitor<MESH::MyMeshData, MESH::MyVertexData, MESH::EdgeData, MESH::MyFaceData, true>());  
  
  // Instanciation of the SceneSaver.
  X3D::SceneSaver *saver = X3DProcessor::getInstanceOf<X3D::SceneSaver>();
  
  // Loads the scene.
  X3D::Scene *s = loader->load(argv[1], false);
  MESH::Scene *ms = meshbuilder->build(s);
  X3D::Scene *ss = meshx3dbuilder->build(ms);
  saver->saveAs(ss, "myfile.x3d");
  
  // removes all instances.
  X3DProcessor::removeAllInstances();  
  X3DLoader::removeInstanceOf<X3D::Loader>();
    
  return 1;
}
