#include "MESH_MyStructureComputerCoreVisitor.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace MESH {

MyStructureComputerCoreVisitor::MyStructureComputerCoreVisitor()
{
  // Enter functions.
  defineEnterFunction(&MyStructureComputerCoreVisitor::enterMesh);
  defineEnterFunction(&MyStructureComputerCoreVisitor::enterTransform);
  
  // Leave function.
  defineLeaveFunction(&MyStructureComputerCoreVisitor::leaveX3DGroupingNode);
  
  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<MyStructureComputerStateVariables>();
}

void MyStructureComputerCoreVisitor::enterMesh(DefMesh *M) const
{
  SFMatrix34f T = stateVariables->getMatrix();

  // Beginning a new Mesh by memorizing the current vertex index.
  stateVariables->beginNewMesh();
  
  // filling the vertices.
  for (DefMesh::MFVertex::const_iterator it = M->getVertices().begin(); it != M->getVertices().end(); ++it)
  {
    SFPoint3f P = T*(*it)->data().getPoint();
    stateVariables->addVertex(P.x, P.y, P.z);
  }
  
  // filling the faces.
  for (DefMesh::MFFace::const_iterator it = M->getFaces().begin(); it != M->getFaces().end(); ++it)
  {
    const DefSFFace::MFEdge &edges = (*it)->getEdges();
    list<unsigned int> indexList;
    for (DefSFFace::MFEdge::const_iterator itEdge = edges.begin(); itEdge != edges.end(); ++itEdge)
      indexList.push_back((*itEdge)->getFromVertex()->getIndex());
    
    stateVariables->addFace(indexList);
  }  
}

void MyStructureComputerCoreVisitor::enterTransform(Transform *T) const
{
  stateVariables->pushMatrix(T->getTransform());
}

void MyStructureComputerCoreVisitor::leaveX3DGroupingNode(X3DGroupingNode *) const
{
  stateVariables->popMatrix();
}

}
}
